@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2019 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2020 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
@@ -25,7 +25,8 @@ std::vector<std::pair<std::string, std::string> >
 GetShortcutsUsage(const std::string &ctrl = "");
 std::vector<std::pair<std::string, std::string> > GetMouseUsage();
 void PrintUsage(const std::string &name);
-void PrintInfo();
+std::vector<std::string> GetBuildInfo();
+void PrintBuildInfo();
 void GetOptions(int argc, char *argv[], bool readConfigFiles, bool exitOnError);
 
 #endif
