@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2019 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2020 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
@@ -39,7 +39,7 @@
 #include "Plugin.h"
 #include "PluginManager.h"
 #include "OpenFile.h"
-#include "WinIcon.h"
+#include "XpmIcon.h"
 #include "Options.h"
 #include "CommandLine.h"
 #include "Context.h"
@@ -50,7 +50,6 @@
 #if defined(HAVE_TOUCHBAR)
 #include "touchBar.h"
 #endif
-
 #if defined(HAVE_3M)
 #include "3M.h"
 #endif
@@ -362,34 +361,6 @@ static void gamepad_handler(void *data)
   }
 }
 
-static void error_handler(const char *fmt, ...)
-{
-  char str[5000];
-  va_list args;
-  va_start(args, fmt);
-  vsnprintf(str, sizeof(str), fmt, args);
-  va_end(args);
-  if(!strcmp(str, "Insufficient GL support")) { // this should be fatal
-    CTX::instance()->terminal = 1;
-    Msg::Error("%s (FLTK internal error)", str);
-    Msg::Fatal("Your system does not seem to support OpenGL - aborting");
-  }
-  else {
-    Msg::Error("%s (FLTK internal error)", str);
-  }
-}
-
-static void fatal_error_handler(const char *fmt, ...)
-{
-  char str[5000];
-  va_list args;
-  va_start(args, fmt);
-  vsnprintf(str, sizeof(str), fmt, args);
-  va_end(args);
-  CTX::instance()->terminal = 1;
-  Msg::Fatal("%s (FLTK internal error)", str);
-}
-
 void FlGui::applyColorScheme(bool redraw)
 {
   static int first = true;
@@ -450,10 +421,47 @@ void FlGui::applyColorScheme(bool redraw)
   }
 }
 
-FlGui::FlGui(int argc, char **argv)
+static void default_error_handler(const char *fmt, ...)
 {
-  Fl::error = error_handler;
-  Fl::fatal = fatal_error_handler;
+  char str[5000];
+  va_list args;
+  va_start(args, fmt);
+  vsnprintf(str, sizeof(str), fmt, args);
+  va_end(args);
+  if(!strcmp(str, "Insufficient GL support")) { // this should be fatal
+    CTX::instance()->terminal = 1;
+    Msg::Error("%s (FLTK internal error)", str);
+    Msg::Error("Your system does not seem to support OpenGL - aborting");
+    Msg::Exit(1);
+  }
+  else {
+    Msg::Error("%s (FLTK internal error)", str);
+  }
+}
+
+static void default_fatal_error_handler(const char *fmt, ...)
+{
+  char str[5000];
+  va_list args;
+  va_start(args, fmt);
+  vsnprintf(str, sizeof(str), fmt, args);
+  va_end(args);
+  Msg::Error("%s (FLTK internal error)", str);
+  Msg::Exit(1);
+}
+
+FlGui::FlGui(int argc, char **argv, bool quitShouldExit,
+             void (*error_handler)(const char *fmt, ...))
+  : _quitShouldExit(quitShouldExit)
+{
+  if(error_handler) {
+    Fl::error = error_handler;
+    Fl::fatal = error_handler;
+  }
+  else {
+    Fl::error = default_error_handler;
+    Fl::fatal = default_fatal_error_handler;
+  }
 
 #if defined(__APPLE__)
   // the defaults use %@, which leads to (lowercase) gmsh
@@ -494,7 +502,6 @@ FlGui::FlGui(int argc, char **argv)
   if(CTX::instance()->guiTheme.size())
     Fl::scheme(CTX::instance()->guiTheme.c_str());
   Fl_Tooltip::size(FL_NORMAL_SIZE);
-  Fl_Tooltip::delay(0.5);
 
   // use retina resolution if available
 #if(FL_MAJOR_VERSION == 1) && (FL_MINOR_VERSION == 3) && (FL_PATCH_VERSION >= 4)
@@ -522,6 +529,10 @@ FlGui::FlGui(int argc, char **argv)
   // load default system icons (for file browser)
   Fl_File_Icon::load_system_icons();
 
+  // FLTK >= 1.3.3 allows to set a default global window icon
+  Fl_RGB_Image icon(&gmsh_icon_pixmap);
+  Fl_Window::default_icon(&icon);
+
   // add callback to respond to Mac Finder
 #if defined(__APPLE__)
   fl_open_callback(OpenProjectMacFinder);
@@ -537,28 +548,6 @@ FlGui::FlGui(int argc, char **argv)
   graph.push_back(
     new graphicWindow(true, CTX::instance()->numTiles,
                       CTX::instance()->detachedMenu ? true : false));
-#if defined(WIN32)
-  graph[0]->getWindow()->icon(
-    (const void *)LoadIcon(fl_display, MAKEINTRESOURCE(IDI_ICON)));
-#elif defined(__APPLE__)
-  // nothing to do here
-#else
-  fl_open_display();
-  static const char *gmsh32x32 =
-    "\x00\x00\x00\x00\x00\x80\x01\x00\x00\x40\x03\x00\
-    \x00\x40\x03\x00\x00\x20\x07\x00\x00\x20\x07\x00\
-    \x00\x10\x0f\x00\x00\x10\x0f\x00\x00\x08\x1f\x00\
-    \x00\x08\x1f\x00\x00\x04\x3f\x00\x00\x04\x3f\x00\
-    \x00\x02\x7f\x00\x00\x02\x7f\x00\x00\x01\xff\x00\
-    \x00\x01\xff\x00\x80\x00\xff\x01\x80\x00\xff\x01\
-    \x40\x00\xff\x03\x40\x00\xff\x03\x20\x00\xff\x07\
-    \x20\x00\xff\x07\x10\x00\xff\x0f\x10\x00\xff\x0f\
-    \x08\x00\xff\x1f\x08\x00\xff\x1f\x04\x40\xfd\x3f\
-    \x04\xa8\xea\x3f\x02\x55\x55\x7f\xa2\xaa\xaa\x7a\
-    \xff\xff\xff\xff\x00\x00\x00\x00";
-  graph[0]->getWindow()->icon((const char *)XCreateBitmapFromData(
-    fl_display, DefaultRootWindow(fl_display), gmsh32x32, 32, 32));
-#endif
 
   graph[0]->getWindow()->show(argc > 0 ? 1 : 0, argv);
   if(graph[0]->getMenuWindow()) graph[0]->getMenuWindow()->show();
@@ -595,9 +584,6 @@ FlGui::FlGui(int argc, char **argv)
   }
   fullscreen->mode(mode);
   fullscreen->end();
-#if !defined(__APPLE__)
-  fullscreen->icon(graph[0]->getWindow()->icon());
-#endif
 
   // create all other windows
   options = new optionWindow(CTX::instance()->deltaFontSize);
@@ -631,19 +617,39 @@ FlGui::FlGui(int argc, char **argv)
 #endif
 }
 
+FlGui::~FlGui()
+{
+  for(std::size_t i = 0; i < graph.size(); i++) delete graph[i];
+  delete options;
+  delete fields;
+  delete plugins;
+  delete stats;
+  delete visibility;
+  delete highordertools;
+  delete clipping;
+  delete manip;
+  delete elementaryContext;
+  delete transformContext;
+  delete physicalContext;
+  delete meshContext;
+  delete help;
+  delete fullscreen;
+}
+
 bool FlGui::available() { return _instance != 0; }
 
-FlGui *FlGui::instance(int argc, char **argv)
+FlGui *FlGui::instance(int argc, char **argv, bool quitShouldExit,
+                       void (*error_handler)(const char *fmt, ...))
 {
   if(!_instance) {
-    _instance = new FlGui(argc, argv);
+    _instance = new FlGui(argc, argv, quitShouldExit, error_handler);
     // set all options in the new GUI
     InitOptionsGUI(0);
     // say welcome!
     Msg::StatusBar(false, "Gmsh %s", GetGmshVersion());
     // log the following for bug reports
     Msg::Direct("-------------------------------------------------------");
-    PrintInfo();
+    PrintBuildInfo();
     Msg::Direct("-------------------------------------------------------");
     // update views (in case the GUI is created after some data has been loaded)
     _instance->updateViews(true, true);
@@ -654,6 +660,13 @@ FlGui *FlGui::instance(int argc, char **argv)
   return _instance;
 }
 
+void FlGui::destroy()
+{
+  if(!_instance) return;
+  delete _instance;
+  _instance = 0;
+}
+
 int FlGui::run()
 {
   // draw the scene
@@ -677,7 +690,8 @@ int FlGui::testGlobalShortcuts(int event)
     geometry_reload_cb(0, 0);
     status = 1;
   }
-  if(Fl::test_shortcut(FL_CTRL + '0') || Fl::test_shortcut(FL_META + '0')) {
+  if(Fl::test_shortcut(FL_CTRL + '0') || Fl::test_shortcut(FL_META + '0') ||
+     Fl::test_shortcut('9')) { // for Bruno
     onelab_reload_cb(0, 0);
     status = 1;
   }
@@ -1196,6 +1210,7 @@ char FlGui::selectEntity(int type)
 
 void FlGui::setStatus(const std::string &msg, bool opengl)
 {
+  if(Msg::GetThreadNum() > 0) return;
   if(!opengl) {
     _lastStatus = msg;
     static char buff[1024];
@@ -1237,6 +1252,7 @@ void FlGui::setStatus(const std::string &msg, bool opengl)
 
 void FlGui::setLastStatus(int col)
 {
+  if(Msg::GetThreadNum() > 0) return;
   for(std::size_t i = 0; i < graph.size(); i++) {
     if(col >= 0 && graph[0]->getMessageHeight() < FL_NORMAL_SIZE) {
       if(CTX::instance()->guiColorScheme) // dark
@@ -1255,6 +1271,7 @@ void FlGui::setLastStatus(int col)
 void FlGui::setProgress(const std::string &msg, double val, double min,
                         double max)
 {
+  if(Msg::GetThreadNum() > 0) return;
   for(std::size_t i = 0; i < FlGui::instance()->graph.size(); i++) {
     if(FlGui::instance()->graph[i]->getProgress()->value() != val)
       FlGui::instance()->graph[i]->getProgress()->value(val);
