@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2019 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2020 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
@@ -6,6 +6,7 @@
 #ifndef CONTEXT_WINDOW_H
 #define CONTEXT_WINDOW_H
 
+#include <FL/Fl.H>
 #include <FL/Fl_Window.H>
 #include <FL/Fl_Input.H>
 #include <FL/Fl_Input_Choice.H>
@@ -26,6 +27,7 @@ public:
 
 public:
   elementaryContextWindow(int deltaFontSize = 0);
+  ~elementaryContextWindow() { Fl::delete_widget(win); }
   void show(int pane);
   void hide() { win->hide(); }
   void updatePoint(double pt[3], int which);
@@ -43,6 +45,7 @@ public:
 
 public:
   transformContextWindow(int deltaFontSize = 0);
+  ~transformContextWindow() { Fl::delete_widget(win); }
   void show(int pane, bool extrude = false, bool selection = true);
   void hide() { win->hide(); }
 };
@@ -59,6 +62,7 @@ public:
 
 public:
   physicalContextWindow(int deltaFontSize = 0);
+  ~physicalContextWindow() { Fl::delete_widget(win); }
   void show(bool remove);
   void hide() { win->hide(); }
 };
@@ -72,6 +76,7 @@ public:
 
 public:
   meshContextWindow(int deltaFontSize = 0);
+  ~meshContextWindow() { Fl::delete_widget(win); }
   void show(int pane);
   void hide() { win->hide(); }
 };
