@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2019 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2020 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
@@ -23,11 +23,11 @@
 // TODO C++11 remove this nasty stuff
 #if __cplusplus >= 201103L
 #include <unordered_map>
-#define hashmapMFace std::unordered_map<MFace, int, Hash_Face, Equal_Face>
-#define hashmapMEdge std::unordered_map<MEdge, int, Hash_Edge, Equal_Edge>
+#define hashmapMFace std::unordered_map<MFace, int, MFaceHash, MFaceEqual>
+#define hashmapMEdge std::unordered_map<MEdge, int, MEdgeHash, MEdgeEqual>
 #else
-#define hashmapMFace std::map<MFace, int, Less_Face>
-#define hashmapMEdge std::map<MEdge, int, Less_Edge>
+#define hashmapMFace std::map<MFace, int, MFaceLessThan>
+#define hashmapMEdge std::map<MEdge, int, MEdgeLessThan>
 #endif
 
 template <class scalar> class simpleFunction;
@@ -35,9 +35,9 @@ template <class scalar> class simpleFunction;
 class GEO_Internals;
 class OCC_Internals;
 class ACIS_Internals;
+class Parasolid_Internals;
 class smooth_normals;
 class FieldManager;
-class CGNSOptions;
 class gLevelset;
 class discreteFace;
 class discreteRegion;
@@ -49,10 +49,10 @@ private:
   std::multimap<std::pair<const std::vector<int>, const std::vector<int> >,
                 std::pair<const std::string, const std::vector<int> > >
     _homologyRequests;
-  std::set<GRegion *, GEntityLessThan> _chainRegions;
-  std::set<GFace *, GEntityLessThan> _chainFaces;
-  std::set<GEdge *, GEntityLessThan> _chainEdges;
-  std::set<GVertex *, GEntityLessThan> _chainVertices;
+  std::set<GRegion *, GEntityPtrLessThan> _chainRegions;
+  std::set<GFace *, GEntityPtrLessThan> _chainFaces;
+  std::set<GEdge *, GEntityPtrLessThan> _chainEdges;
+  std::set<GVertex *, GEntityPtrLessThan> _chainVertices;
   hashmapMEdge _mapEdgeNum;
   hashmapMFace _mapFaceNum;
   // the maximum vertex and element id number in the mesh
@@ -115,12 +115,14 @@ protected:
   // global cache storage of discrete curvatures
   std::map<MVertex *, std::pair<SVector3, SVector3> > _curvatures;
 
-  // Geo (Gmsh native) model internal data
+  // GEO (Gmsh native) model internal data
   GEO_Internals *_geo_internals;
-  // OpenCascade model internal data
+  // OpenCASCADE model internal data
   OCC_Internals *_occ_internals;
   // ACIS model internal data
   ACIS_Internals *_acis_internals;
+  // Parasolid model internal data
+  Parasolid_Internals *_parasolid_internals;
 
   // characteristic length (mesh size) fields
   FieldManager *_fields;
@@ -138,10 +140,10 @@ protected:
 
   // the sets of geometrical regions, faces, edges and vertices in the
   // model
-  std::set<GRegion *, GEntityLessThan> regions;
-  std::set<GFace *, GEntityLessThan> faces;
-  std::set<GEdge *, GEntityLessThan> edges;
-  std::set<GVertex *, GEntityLessThan> vertices;
+  std::set<GRegion *, GEntityPtrLessThan> regions;
+  std::set<GFace *, GEntityPtrLessThan> faces;
+  std::set<GEdge *, GEntityPtrLessThan> edges;
+  std::set<GVertex *, GEntityPtrLessThan> vertices;
 
   // map between the pair <dimension, elementary or physical number>
   // and an optional associated name
@@ -151,18 +153,6 @@ protected:
   std::size_t _numPartitions;
 
 protected:
-  void _createGEOInternals();
-  void _deleteGEOInternals();
-
-  void _deleteOCCInternals();
-  void _resetOCCInternals();
-
-  void _deleteACISInternals();
-
-  // CGNS helpers
-  int _readCGNSStructured(const std::string &name);
-  int _readCGNSUnstructured(const std::string &name);
-
   // store the elements given in the map (indexed by elementary region
   // number) into the model, creating discrete geometrical entities on
   // the fly if needed
@@ -187,15 +177,15 @@ protected:
 
 public:
   // region, face, edge and vertex iterators
-  typedef std::set<GRegion *, GEntityLessThan>::iterator riter;
-  typedef std::set<GFace *, GEntityLessThan>::iterator fiter;
-  typedef std::set<GEdge *, GEntityLessThan>::iterator eiter;
-  typedef std::set<GVertex *, GEntityLessThan>::iterator viter;
+  typedef std::set<GRegion *, GEntityPtrLessThan>::iterator riter;
+  typedef std::set<GFace *, GEntityPtrLessThan>::iterator fiter;
+  typedef std::set<GEdge *, GEntityPtrLessThan>::iterator eiter;
+  typedef std::set<GVertex *, GEntityPtrLessThan>::iterator viter;
 
-  typedef std::set<GRegion *, GEntityLessThan>::const_iterator const_riter;
-  typedef std::set<GFace *, GEntityLessThan>::const_iterator const_fiter;
-  typedef std::set<GEdge *, GEntityLessThan>::const_iterator const_eiter;
-  typedef std::set<GVertex *, GEntityLessThan>::const_iterator const_viter;
+  typedef std::set<GRegion *, GEntityPtrLessThan>::const_iterator const_riter;
+  typedef std::set<GFace *, GEntityPtrLessThan>::const_iterator const_fiter;
+  typedef std::set<GEdge *, GEntityPtrLessThan>::const_iterator const_eiter;
+  typedef std::set<GVertex *, GEntityPtrLessThan>::const_iterator const_viter;
 
   // elementary/physical name iterator
   typedef std::map<std::pair<int, int>, std::string>::iterator piter;
@@ -229,8 +219,56 @@ public:
   // get/set global vertex/element num
   std::size_t getMaxVertexNumber() const { return _maxVertexNum; }
   std::size_t getMaxElementNumber() const { return _maxElementNum; }
-  void setMaxVertexNumber(std::size_t num) { _maxVertexNum = num; }
-  void setMaxElementNumber(std::size_t num) { _maxElementNum = num; }
+  void setMaxVertexNumber(std::size_t num)
+  {
+#if defined(_OPENMP)
+#pragma omp atomic write
+#endif
+    _maxVertexNum = _maxVertexNum > num ? _maxVertexNum : num;
+  }
+  void setMaxElementNumber(std::size_t num)
+  {
+#if defined(_OPENMP)
+#pragma omp atomic write
+#endif
+    _maxElementNum = _maxElementNum > num ? _maxElementNum : num;
+  }
+
+  // increment and get global vertex/element num
+  std::size_t incrementAndGetMaxVertexNumber()
+  {
+    std::size_t _myVertexNum;
+#if defined(_OPENMP)
+#pragma omp atomic capture
+#endif
+    {
+      ++_maxVertexNum;
+      _myVertexNum = _maxVertexNum;
+    }
+    return _myVertexNum;
+  }
+  std::size_t incrementAndGetMaxElementNumber()
+  {
+    std::size_t _myElementNum;
+#if defined(_OPENMP)
+#pragma omp atomic capture
+#endif
+    {
+      ++_maxElementNum;
+      _myElementNum = _maxElementNum;
+    }
+    return _myElementNum;
+  }
+
+  // decrement global vertex num
+  void decrementMaxVertexNumber()
+  {
+#if defined(_OPENMP)
+#pragma omp atomic update
+#endif
+    --_maxVertexNum;
+  }
+
   void checkPointMaxNumbers()
   {
     _checkPointedMaxVertexNum = _maxVertexNum;
@@ -244,7 +282,9 @@ public:
 
   // number the edges
   int addMEdge(const MEdge &edge);
-  //number the faces
+  // get the edge number
+  int getEdgeNumber(const MEdge &edge);
+  // number the faces
   int addMFace(const MFace &face);
 
   // renumber mesh vertices and elements in a continuous sequence (this
@@ -257,6 +297,7 @@ public:
   void destroyMeshCaches();
   // delete the mesh stored in entities and call destroMeshCaches
   void deleteMesh();
+  void deleteMesh(const std::vector<GEntity *> &entities);
   // delete the vertex arrays used for efficient mesh drawing
   void deleteVertexArrays();
 
@@ -266,10 +307,19 @@ public:
   void pruneMeshVertexAssociations();
 
   // access internal CAD representations
-  GEO_Internals *getGEOInternals() { return _geo_internals; }
+  void createGEOInternals();
   void createOCCInternals();
+  void createACISInternals();
+  void createParasolidInternals();
   OCC_Internals *getOCCInternals() { return _occ_internals; }
+  GEO_Internals *getGEOInternals() { return _geo_internals; }
   ACIS_Internals *getACISInternals() { return _acis_internals; }
+  Parasolid_Internals *getParasolidInternals() { return _parasolid_internals; }
+  void deleteGEOInternals();
+  void deleteOCCInternals();
+  void resetOCCInternals();
+  void deleteACISInternals();
+  void deleteParasolidInternals();
 
   // access characteristic length (mesh size) fields
   FieldManager *getFields() { return _fields; }
@@ -319,10 +369,16 @@ public:
   const_viter lastVertex() const { return vertices.end(); }
 
   // get the set of entities
-  std::set<GRegion *, GEntityLessThan> getRegions() const { return regions; };
-  std::set<GFace *, GEntityLessThan> getFaces() const { return faces; };
-  std::set<GEdge *, GEntityLessThan> getEdges() const { return edges; };
-  std::set<GVertex *, GEntityLessThan> getVertices() const { return vertices; };
+  std::set<GRegion *, GEntityPtrLessThan> getRegions() const
+  {
+    return regions;
+  };
+  std::set<GFace *, GEntityPtrLessThan> getFaces() const { return faces; };
+  std::set<GEdge *, GEntityPtrLessThan> getEdges() const { return edges; };
+  std::set<GVertex *, GEntityPtrLessThan> getVertices() const
+  {
+    return vertices;
+  };
 
   // find the entity with the given tag
   GRegion *getRegionByTag(int n) const;
@@ -397,7 +453,7 @@ public:
   piter lastElementaryName() { return _elementaryNames.end(); }
 
   // get the number of physical names
-  int numPhysicalNames() const { return _physicalNames.size(); }
+  int numPhysicalNames() const { return (int)_physicalNames.size(); }
 
   // get iterators to the last physical name of each dimension
   void getInnerPhysicalNamesIterators(std::vector<piter> &iterators);
@@ -465,8 +521,8 @@ public:
   std::size_t getNumMeshElements(unsigned c[6]);
 
   // access a mesh element by coordinates (using an octree search)
-  MElement *getMeshElementByCoord(SPoint3 &p, SPoint3 &param,
-                                  int dim = -1, bool strict = true);
+  MElement *getMeshElementByCoord(SPoint3 &p, SPoint3 &param, int dim = -1,
+                                  bool strict = true);
   std::vector<MElement *> getMeshElementsByCoord(SPoint3 &p, int dim = -1,
                                                  bool strict = true);
 
@@ -483,6 +539,10 @@ public:
   // recompute _vertexVectorCache if there is a dense vertex numbering or
   // _vertexMapCache if not.
   void rebuildMeshVertexCache(bool onlyIfNecessary = false);
+  
+  // recompute _elementVectorCache if there is a dense element numbering or
+  // _elementMapCache if not.
+  void rebuildMeshElementCache(bool onlyIfNecessary = false);
 
   // access a mesh vertex by tag, using the vertex cache
   MVertex *getMeshVertexByTag(int n);
@@ -518,12 +578,13 @@ public:
     return _lastMeshVertexError;
   }
 
-  // delete all invisble mesh elements
+  // delete or reverse all invisble mesh elements
   void removeInvisibleElements();
+  void reverseInvisibleElements();
 
   // the list of partitions
   std::size_t getNumPartitions() const { return _numPartitions; }
-  void setNumPartitions(unsigned int npart) { _numPartitions = npart; }
+  void setNumPartitions(std::size_t npart) { _numPartitions = npart; }
 
   // partition the mesh
   int partitionMesh(int num);
@@ -548,17 +609,18 @@ public:
   // remove duplicate mesh vertices
   int removeDuplicateMeshVertices(double tolerance);
 
-  // create a topology from the mesh if necessary, move the mesh of discrete
-  // entities to a geometry container, compute a parametrization for the
-  // discrete entities
-  void createGeometryOfDiscreteEntities();
+  // create a geometry (i.e. a parametrization for curves and surfaces) for the
+  // given discrete entities (or all of them if dimTags is empty)
+  void createGeometryOfDiscreteEntities(const std::vector<std::pair<int, int> >
+                                        &dimTags = std::vector<std::pair<int, int> >());
 
-  // create topology from mesh
-  void createTopologyFromMeshNew();
-  void createTopologyFromMesh();
+  // make discrete entities simply connected
   void makeDiscreteRegionsSimplyConnected();
   void makeDiscreteFacesSimplyConnected();
 
+  // create topology from mesh
+  void createTopologyFromMesh();
+
   // align periodic boundaries
   void alignPeriodicBoundaries();
 
@@ -589,13 +651,11 @@ public:
   int setOrderN(int order, int linear, int incomplete);
 
   // refine the mesh by splitting all elements
-  int refineMesh(int linear, bool barycentric = false);
+  int refineMesh(int linear, bool splitIntoQuads = false,
+                 bool splitIntoHexas = false, bool barycentric = false);
 
   // optimize the mesh
-  int optimizeMesh(const std::string &how, bool force = false);
-
-  // smooth the mesh
-  int smoothMesh();
+  int optimizeMesh(const std::string &how, bool force = false, int niter = 1);
 
   // recombine the mesh
   int recombineMesh();
@@ -605,7 +665,7 @@ public:
 
   // reclassify a surface mesh, using an angle threshold to tag edges and faces
   void classifySurfaces(double angleThreshold, bool includeBoundary,
-                        bool forReparametrization);
+                        bool forReparametrization, double curveAngleThreshold);
 
   // build a new GModel by cutting the elements crossed by the levelset ls
   // if cutElem is set to false, split the model without cutting the elements
@@ -655,6 +715,12 @@ public:
   // ACIS Model
   int readACISSAT(const std::string &name);
 
+  // Parasolid Model
+  int readParasolidXMT(const std::string &name);
+  int writeParasolidXMT(const std::string &name);
+  int readParasolidSTEP(const std::string &name);
+  int writeParasolidSTEP(const std::string &name);
+
   // Gmsh mesh file format
   int readMSH(const std::string &name);
   int writeMSH(const std::string &name, double version = 2.2,
@@ -683,6 +749,11 @@ public:
                bool saveAll = false, double scalingFactor = 1.0,
                int oneSolidPerSurface = 0);
 
+  // X3D (only output from OCCT's triangulation)
+  int writeX3D(const std::string &name, bool saveAll = false,
+               double scalingFactor = 1.0, int x3dsurfaces = 1,
+               int x3dedges = 0, int x3dvertices = 0);
+
   // PLY(2) format (ascii text format)
   int readPLY(const std::string &name);
   int readPLY2(const std::string &name);
@@ -720,9 +791,11 @@ public:
                double scalingFactor = 1.0);
 
   // CFD General Notation System files
-  int readCGNS(const std::string &name);
-  int writeCGNS(const std::string &name, int zoneDefinition,
-                const CGNSOptions &options, double scalingFactor = 1.0);
+  int readCGNS(const std::string &name,
+               std::vector<std::vector<MVertex *> > &vertPerZone,
+               std::vector<std::vector<MElement *> > &eltPerZone);
+  int writeCGNS(const std::string &name, bool saveAll = false,
+                double scalingFactor = 1.0);
 
   // Med "Modele d'Echange de Donnees" file format (the static routine
   // is allowed to load multiple models/meshes)
@@ -753,7 +826,8 @@ public:
 
   // Abaqus
   int writeINP(const std::string &name, bool saveAll = false,
-               bool saveGroupsOfNodes = false, double scalingFactor = 1.0);
+               int saveGroupsOfElements = 1, int saveGroupsOfNodes = 0,
+               double scalingFactor = 1.0);
 
   // LSDYNA
   int writeKEY(const std::string &name, int saveAll = 0,
