@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2019 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2020 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
@@ -10,6 +10,7 @@
 #include "GModel.h"
 #include "GModelIO_GEO.h"
 #include "Geo.h"
+#include "GeoInterpolation.h"
 #include "OS.h"
 #include "OpenFile.h"
 #include "Numeric.h"
@@ -152,7 +153,7 @@ bool GEO_Internals::addLine(int &tag, const std::vector<int> &pointTags)
     return false;
   }
   if(pointTags.size() < 2) {
-    Msg::Error("Line curve requires 2 points");
+    Msg::Error("Line requires 2 points");
     return false;
   }
   if(tag < 0) tag = getMaxTag(1) + 1;
@@ -161,12 +162,13 @@ bool GEO_Internals::addLine(int &tag, const std::vector<int> &pointTags)
     int t = pointTags[i];
     List_Add(tmp, &t);
   }
-  Curve *c = CreateCurve(tag, MSH_SEGM_LINE, 1, tmp, NULL, -1, -1, 0., 1.);
+  bool ok = true;
+  Curve *c = CreateCurve(tag, MSH_SEGM_LINE, 1, tmp, NULL, -1, -1, 0., 1., ok);
   Tree_Add(Curves, &c);
   CreateReversedCurve(c);
   List_Delete(tmp);
   _changed = true;
-  return true;
+  return ok;
 }
 
 bool GEO_Internals::addCircleArc(int &tag, int startTag, int centerTag,
@@ -181,7 +183,8 @@ bool GEO_Internals::addCircleArc(int &tag, int startTag, int centerTag,
   List_Add(tmp, &startTag);
   List_Add(tmp, &centerTag);
   List_Add(tmp, &endTag);
-  Curve *c = CreateCurve(tag, MSH_SEGM_CIRC, 2, tmp, NULL, -1, -1, 0., 1.);
+  bool ok = true;
+  Curve *c = CreateCurve(tag, MSH_SEGM_CIRC, 2, tmp, NULL, -1, -1, 0., 1., ok);
   if(nx || ny || nz) {
     c->Circle.n[0] = nx;
     c->Circle.n[1] = ny;
@@ -198,7 +201,7 @@ bool GEO_Internals::addCircleArc(int &tag, int startTag, int centerTag,
   }
   List_Delete(tmp);
   _changed = true;
-  return true;
+  return ok;
 }
 
 bool GEO_Internals::addEllipseArc(int &tag, int startTag, int centerTag,
@@ -215,7 +218,8 @@ bool GEO_Internals::addEllipseArc(int &tag, int startTag, int centerTag,
   List_Add(tmp, &centerTag);
   List_Add(tmp, &majorTag);
   List_Add(tmp, &endTag);
-  Curve *c = CreateCurve(tag, MSH_SEGM_ELLI, 2, tmp, NULL, -1, -1, 0., 1.);
+  bool ok = true;
+  Curve *c = CreateCurve(tag, MSH_SEGM_ELLI, 2, tmp, NULL, -1, -1, 0., 1., ok);
   if(nx || ny || nz) {
     c->Circle.n[0] = nx;
     c->Circle.n[1] = ny;
@@ -232,7 +236,7 @@ bool GEO_Internals::addEllipseArc(int &tag, int startTag, int centerTag,
   }
   List_Delete(tmp);
   _changed = true;
-  return true;
+  return ok;
 }
 
 bool GEO_Internals::addSpline(int &tag, const std::vector<int> &pointTags)
@@ -251,12 +255,13 @@ bool GEO_Internals::addSpline(int &tag, const std::vector<int> &pointTags)
     int t = pointTags[i];
     List_Add(tmp, &t);
   }
-  Curve *c = CreateCurve(tag, MSH_SEGM_SPLN, 3, tmp, NULL, -1, -1, 0., 1.);
+  bool ok = true;
+  Curve *c = CreateCurve(tag, MSH_SEGM_SPLN, 3, tmp, NULL, -1, -1, 0., 1., ok);
   Tree_Add(Curves, &c);
   CreateReversedCurve(c);
   List_Delete(tmp);
   _changed = true;
-  return true;
+  return ok;
 }
 
 bool GEO_Internals::addBezier(int &tag, const std::vector<int> &pointTags)
@@ -275,12 +280,13 @@ bool GEO_Internals::addBezier(int &tag, const std::vector<int> &pointTags)
     int t = pointTags[i];
     List_Add(tmp, &t);
   }
-  Curve *c = CreateCurve(tag, MSH_SEGM_BEZIER, 2, tmp, NULL, -1, -1, 0., 1.);
+  bool ok = true;
+  Curve *c = CreateCurve(tag, MSH_SEGM_BEZIER, 2, tmp, NULL, -1, -1, 0., 1., ok);
   Tree_Add(Curves, &c);
   CreateReversedCurve(c);
   List_Delete(tmp);
   _changed = true;
-  return true;
+  return ok;
 }
 
 bool GEO_Internals::addBSpline(int &tag, const std::vector<int> &pointTags,
@@ -300,9 +306,10 @@ bool GEO_Internals::addBSpline(int &tag, const std::vector<int> &pointTags,
     int t = pointTags[i];
     List_Add(tmp, &t);
   }
-  Curve *c;
+  Curve *c = NULL;
+  bool ok = true;
   if(seqknots.empty()) {
-    c = CreateCurve(tag, MSH_SEGM_BSPLN, 2, tmp, NULL, -1, -1, 0., 1.);
+    c = CreateCurve(tag, MSH_SEGM_BSPLN, 2, tmp, NULL, -1, -1, 0., 1., ok);
   }
   else {
     int order = seqknots.size() - pointTags.size() - 1;
@@ -311,13 +318,70 @@ bool GEO_Internals::addBSpline(int &tag, const std::vector<int> &pointTags,
       double d = seqknots[i];
       List_Add(knotsList, &d);
     }
-    c = CreateCurve(tag, MSH_SEGM_NURBS, order, tmp, knotsList, -1, -1, 0., 1.);
+    c = CreateCurve(tag, MSH_SEGM_NURBS, order, tmp, knotsList, -1, -1, 0., 1., ok);
   }
   Tree_Add(Curves, &c);
   CreateReversedCurve(c);
   List_Delete(tmp);
   _changed = true;
-  return true;
+  return ok;
+}
+
+bool GEO_Internals::_addCompoundSpline(int &tag, const std::vector<int> &curveTags,
+                                       int numIntervals, bool bspline)
+{
+  if(tag >= 0 && FindCurve(tag)) {
+    Msg::Error("GEO curve with tag %d already exists", tag);
+    return false;
+  }
+  if(curveTags.empty()) {
+    Msg::Error("Compound spline curve requires at least 1 input curve");
+    return false;
+  }
+  if(numIntervals < 0) {
+    Msg::Error("Negative number of intervals in compound spline");
+    return false;
+  }
+  if(tag < 0) tag = getMaxTag(1) + 1;
+
+  List_T *tmp = List_Create((numIntervals + 1) * curveTags.size(), 2, sizeof(int));
+  for(std::size_t i = 0; i < curveTags.size(); i++) {
+    Curve *c = FindCurve(curveTags[i]);
+    if(i == 0 && c->beg) List_Add(tmp, &c->beg->Num);
+    for(int j = 1; j < numIntervals; j++) {
+      double u = (double)j / (double)(numIntervals);
+      Vertex V = InterpolateCurve(c, u, 0);
+      double lc = (1 - u) * c->beg->lc + u * c->end->lc;
+      int tag = getMaxTag(0) + 1;
+      Vertex *v = CreateVertex(tag, V.Pos.X, V.Pos.Y, V.Pos.Z, lc, 1.0);
+      Tree_Add(Points, &v);
+      List_Add(tmp, &v->Num);
+    }
+    if(c->end) List_Add(tmp, &c->end->Num);
+  }
+  bool ok = true;
+  Curve *c;
+  if(bspline)
+    c = CreateCurve(tag, MSH_SEGM_BSPLN, 2, tmp, NULL, -1, -1, 0., 1., ok);
+  else //often too oscillatory for non-uniform distribution of control points
+    c = CreateCurve(tag, MSH_SEGM_SPLN, 3, tmp, NULL, -1, -1, 0., 1., ok);
+  Tree_Add(Curves, &c);
+  CreateReversedCurve(c);
+  List_Delete(tmp);
+  _changed = true;
+  return ok;
+}
+
+bool GEO_Internals::addCompoundSpline(int &tag, const std::vector<int> &curveTags,
+                                      int numIntervals)
+{
+  return _addCompoundSpline(tag, curveTags, numIntervals, false);
+}
+
+bool GEO_Internals::addCompoundBSpline(int &tag, const std::vector<int> &curveTags,
+                                      int numIntervals)
+{
+  return _addCompoundSpline(tag, curveTags, numIntervals, true);
 }
 
 bool GEO_Internals::addLineLoop(int &tag, const std::vector<int> &curveTags)
@@ -357,12 +421,12 @@ bool GEO_Internals::addPlaneSurface(int &tag, const std::vector<int> &wireTags)
     List_Add(tmp, &t);
   }
   Surface *s = CreateSurface(tag, MSH_SURF_PLAN);
-  SetSurfaceGeneratrices(s, tmp);
+  bool ok = SetSurfaceGeneratrices(s, tmp);
   List_Delete(tmp);
   EndSurface(s);
   Tree_Add(Surfaces, &s);
   _changed = true;
-  return true;
+  return ok;
 }
 
 bool GEO_Internals::addDiscreteSurface(int &tag)
@@ -413,18 +477,19 @@ bool GEO_Internals::addSurfaceFilling(int &tag,
     List_Add(tmp, &t);
   }
   Surface *s = CreateSurface(tag, type);
-  SetSurfaceGeneratrices(s, tmp);
+  bool ok = SetSurfaceGeneratrices(s, tmp);
   List_Delete(tmp);
   EndSurface(s);
   if(sphereCenterTag >= 0) {
     s->InSphereCenter = FindPoint(sphereCenterTag);
     if(!s->InSphereCenter) {
-      Msg::Error("Unknown sphere center vertex %d", sphereCenterTag);
+      Msg::Error("Unknown sphere center point %d", sphereCenterTag);
+      ok = false;
     }
   }
   Tree_Add(Surfaces, &s);
   _changed = true;
-  return true;
+  return ok;
 }
 
 bool GEO_Internals::addSurfaceLoop(int &tag,
@@ -462,11 +527,11 @@ bool GEO_Internals::addVolume(int &tag, const std::vector<int> &shellTags)
     List_Add(tmp, &t);
   }
   Volume *v = CreateVolume(tag, MSH_VOLUME);
-  SetVolumeSurfaces(v, tmp);
+  bool ok = SetVolumeSurfaces(v, tmp);
   List_Delete(tmp);
   Tree_Add(Volumes, &v);
   _changed = true;
-  return true;
+  return ok;
 }
 
 bool GEO_Internals::_extrude(int mode,
@@ -575,15 +640,16 @@ bool GEO_Internals::_transform(int mode,
     s.Num = tag;
     List_Add(list, &s);
   }
+  bool ok = true;
   switch(mode) {
-  case 0: TranslateShapes(dx, dy, dz, list); break;
-  case 1: RotateShapes(dx, dy, dz, x, y, z, a, list); break;
-  case 2: DilatShapes(x, y, z, a, b, c, list); break;
-  case 3: SymmetryShapes(a, b, c, d, list); break;
+  case 0: ok = TranslateShapes(dx, dy, dz, list); break;
+  case 1: ok = RotateShapes(dx, dy, dz, x, y, z, a, list); break;
+  case 2: ok = DilatShapes(x, y, z, a, b, c, list); break;
+  case 3: ok = SymmetryShapes(a, b, c, d, list); break;
   }
   List_Delete(list);
   _changed = true;
-  return true;
+  return ok;
 }
 
 bool GEO_Internals::translate(const std::vector<std::pair<int, int> > &dimTags,
@@ -621,7 +687,7 @@ bool GEO_Internals::splitCurve(int tag, const std::vector<int> &pointTags,
     List_Add(tmp, &t);
   }
   List_T *curves = List_Create(10, 10, sizeof(Curve *));
-  SplitCurve(tag, tmp, curves);
+  bool ok = SplitCurve(tag, tmp, curves);
   for(int i = 0; i < List_Nbr(curves); i++) {
     Curve *c;
     List_Read(curves, i, &c);
@@ -630,7 +696,7 @@ bool GEO_Internals::splitCurve(int tag, const std::vector<int> &pointTags,
   List_Delete(tmp);
   List_Delete(curves);
   _changed = true;
-  return true;
+  return ok;
 }
 
 bool GEO_Internals::intersectCurvesWithSurface(
@@ -643,7 +709,7 @@ bool GEO_Internals::intersectCurvesWithSurface(
     double d = curveTags[i];
     List_Add(curves, &d);
   }
-  IntersectCurvesWithSurface(curves, surfaceTag, shapes);
+  bool ok = IntersectCurvesWithSurface(curves, surfaceTag, shapes);
   for(int i = 0; i < List_Nbr(shapes); i++) {
     Shape s;
     List_Read(shapes, i, &s);
@@ -654,13 +720,13 @@ bool GEO_Internals::intersectCurvesWithSurface(
     }
   }
   _changed = true;
-  return true;
+  return ok;
 }
 
 bool GEO_Internals::copy(const std::vector<std::pair<int, int> > &inDimTags,
                          std::vector<std::pair<int, int> > &outDimTags)
 {
-  bool ret = true;
+  bool ok = true;
   for(std::size_t i = 0; i < inDimTags.size(); i++) {
     int dim = inDimTags[i].first;
     int tag = inDimTags[i].second;
@@ -668,7 +734,7 @@ bool GEO_Internals::copy(const std::vector<std::pair<int, int> > &inDimTags,
       Vertex *v = FindPoint(tag);
       if(!v) {
         Msg::Error("Unknown GEO point with tag %d", tag);
-        ret = false;
+        ok = false;
       }
       else {
         Vertex *newv = DuplicateVertex(v);
@@ -679,7 +745,7 @@ bool GEO_Internals::copy(const std::vector<std::pair<int, int> > &inDimTags,
       Curve *c = FindCurve(tag);
       if(!c) {
         Msg::Error("Unknown GEO curve with tag %d", tag);
-        ret = false;
+        ok = false;
       }
       else {
         Curve *newc = DuplicateCurve(c);
@@ -690,7 +756,7 @@ bool GEO_Internals::copy(const std::vector<std::pair<int, int> > &inDimTags,
       Surface *s = FindSurface(tag);
       if(!s) {
         Msg::Error("Unknown GEO surface with tag %d", tag);
-        ret = false;
+        ok = false;
       }
       else {
         Surface *news = DuplicateSurface(s);
@@ -701,7 +767,7 @@ bool GEO_Internals::copy(const std::vector<std::pair<int, int> > &inDimTags,
       Volume *v = FindVolume(tag);
       if(!v) {
         Msg::Error("Unknown GEO volume with tag %d", tag);
-        ret = false;
+        ok = false;
       }
       else {
         Volume *newv = DuplicateVolume(v);
@@ -710,7 +776,7 @@ bool GEO_Internals::copy(const std::vector<std::pair<int, int> > &inDimTags,
     }
   }
   _changed = true;
-  return ret;
+  return ok;
 }
 
 bool GEO_Internals::remove(int dim, int tag, bool recursive)
@@ -756,7 +822,7 @@ bool GEO_Internals::modifyPhysicalGroup(int dim, int tag, int op,
     break;
   case 1:
     type = MSH_PHYSICAL_LINE;
-    str = "line";
+    str = "curve";
     break;
   case 2:
     type = MSH_PHYSICAL_SURFACE;
@@ -1083,7 +1149,25 @@ void GEO_Internals::setReverseMesh(int dim, int tag, bool val)
   _changed = true;
 }
 
-void GEO_Internals::synchronize(GModel *model)
+void GEO_Internals::setMeshAlgorithm(int dim, int tag, int val)
+{
+  if(dim == 2) {
+    Surface *s = FindSurface(tag);
+    if(s) s->MeshAlgorithm = val;
+  }
+  _changed = true;
+}
+
+void GEO_Internals::setMeshSizeFromBoundary(int dim, int tag, int val)
+{
+  if(dim == 2) {
+    Surface *s = FindSurface(tag);
+    if(s) s->MeshSizeFromBoundary = val;
+  }
+  _changed = true;
+}
+
+void GEO_Internals::synchronize(GModel *model, bool resetMeshAttributes)
 {
   Msg::Debug("Syncing GEO_Internals with GModel");
 
@@ -1148,7 +1232,7 @@ void GEO_Internals::synchronize(GModel *model)
       else {
         if(v->getNativeType() == GEntity::GmshModel)
           ((gmshVertex *)v)->resetNativePtr(p);
-        v->resetMeshAttributes();
+        if(resetMeshAttributes) v->resetMeshAttributes();
       }
     }
     List_Delete(points);
@@ -1156,7 +1240,6 @@ void GEO_Internals::synchronize(GModel *model)
 
   if(Tree_Nbr(Curves)) {
     List_T *curves = Tree2List(Curves);
-    // generate all curves except compounds
     for(int i = 0; i < List_Nbr(curves); i++) {
       Curve *c;
       List_Read(curves, i, &c);
@@ -1180,7 +1263,7 @@ void GEO_Internals::synchronize(GModel *model)
             else
               ((gmshEdge *)e)->resetNativePtr(c, 0, 0);
           }
-          e->resetMeshAttributes();
+          if(resetMeshAttributes) e->resetMeshAttributes();
         }
         if(c->degenerated) e->setTooSmall(true);
       }
@@ -1201,7 +1284,7 @@ void GEO_Internals::synchronize(GModel *model)
       else {
         if(f->getNativeType() == GEntity::GmshModel)
           ((gmshFace *)f)->resetNativePtr(s);
-        f->resetMeshAttributes();
+        if(resetMeshAttributes) f->resetMeshAttributes();
       }
     }
     List_Delete(surfaces);
@@ -1220,7 +1303,7 @@ void GEO_Internals::synchronize(GModel *model)
       else {
         if(r->getNativeType() == GEntity::GmshModel)
           ((gmshRegion *)r)->resetNativePtr(v);
-        r->resetMeshAttributes();
+        if(resetMeshAttributes) r->resetMeshAttributes();
       }
     }
     List_Delete(volumes);
@@ -1269,7 +1352,7 @@ void GEO_Internals::synchronize(GModel *model)
       case 1: ent = model->getEdgeByTag(tag); break;
       case 2: ent = model->getFaceByTag(tag); break;
       case 3: ent = model->getRegionByTag(tag); break;
-      default: Msg::Error("compound mesh with dimension %d", dim);
+      default: Msg::Error("Compound mesh constraint with dimension %d", dim);
       }
       if(ent) ents.push_back(ent);
     }
@@ -1342,9 +1425,9 @@ gmshSurface *GEO_Internals::newGeometryPolarSphere(int tag, int centerTag,
 
 // GModel interface
 
-void GModel::_createGEOInternals() { _geo_internals = new GEO_Internals; }
+void GModel::createGEOInternals() { _geo_internals = new GEO_Internals; }
 
-void GModel::_deleteGEOInternals()
+void GModel::deleteGEOInternals()
 {
   if(_geo_internals) delete _geo_internals;
   _geo_internals = 0;
@@ -1542,12 +1625,7 @@ int GModel::writeGEO(const std::string &name, bool printLabels,
 
 int GModel::exportDiscreteGEOInternals()
 {
-  int maxv = 1; // FIXME: temorary - see TODO below
-
-  if(_geo_internals) {
-    maxv = _geo_internals->getMaxTag(3);
-    delete _geo_internals;
-  }
+  if(_geo_internals) delete _geo_internals;
   _geo_internals = new GEO_Internals;
 
   for(viter it = firstVertex(); it != lastVertex(); it++) {
@@ -1558,34 +1636,41 @@ int GModel::exportDiscreteGEOInternals()
 
   for(eiter it = firstEdge(); it != lastEdge(); it++) {
     if((*it)->geomType() == GEntity::DiscreteCurve) {
+      bool ok = true;
       Curve *c = CreateCurve((*it)->tag(), MSH_SEGM_DISCRETE, 1, NULL, NULL, -1,
-                             -1, 0., 1.);
-      List_T *points = Tree2List(_geo_internals->Points);
+                             -1, 0., 1., ok);
+      c->Control_Points = List_Create(2, 1, sizeof(Vertex *));
       GVertex *gvb = (*it)->getBeginVertex();
-      GVertex *gve = (*it)->getEndVertex();
-
-      if(!gvb || !gve) {
-        Msg::Error("Discrete curve %d has NULL endpoint(s): %p %p",
-                   (*it)->tag(), gvb, gve);
-      }
-      int nb = 2;
-      c->Control_Points = List_Create(nb, 1, sizeof(Vertex *));
-      for(int i = 0; i < List_Nbr(points); i++) {
-        Vertex *v;
-        List_Read(points, i, &v);
-        if(gvb && gvb->tag() == v->Num) {
+      if(gvb) {
+        Vertex *v = FindPoint(gvb->tag());
+        if(v) {
           List_Add(c->Control_Points, &v);
           c->beg = v;
         }
-        if(gve && gve->tag() == v->Num) {
+        else {
+          Msg::Error("Could not find GEO point %d", gvb->tag());
+        }
+      }
+      else {
+        Msg::Warning("Discrete curve %d has no begin point", (*it)->tag());
+      }
+      GVertex *gve = (*it)->getEndVertex();
+      if(gve) {
+        Vertex *v = FindPoint(gve->tag());
+        if(v) {
           List_Add(c->Control_Points, &v);
           c->end = v;
         }
+        else {
+          Msg::Error("Could not find GEO point %d", gve->tag());
+        }
+      }
+      else {
+        Msg::Warning("Discrete curve %d has no end point", (*it)->tag());
       }
       EndCurve(c);
       Tree_Add(_geo_internals->Curves, &c);
       CreateReversedCurve(c);
-      List_Delete(points);
     }
   }
 
@@ -1593,29 +1678,45 @@ int GModel::exportDiscreteGEOInternals()
     if((*it)->geomType() == GEntity::DiscreteSurface) {
       Surface *s = CreateSurface((*it)->tag(), MSH_SURF_DISCRETE);
       std::vector<GEdge *> const &edges = (*it)->edges();
-      s->Generatrices = List_Create(edges.size(), 1, sizeof(Curve *));
-      List_T *curves = Tree2List(_geo_internals->Curves);
-      Curve *c;
+      s->Generatrices = List_Create(edges.size() + 1, 1, sizeof(Curve *));
       for(std::vector<GEdge *>::const_iterator ite = edges.begin();
           ite != edges.end(); ite++) {
-        for(int i = 0; i < List_Nbr(curves); i++) {
-          List_Read(curves, i, &c);
-          if(c->Num == (*ite)->tag()) { List_Add(s->Generatrices, &c); }
+        Curve *c = FindCurve((*ite)->tag());
+        if(c) {
+          List_Add(s->Generatrices, &c);
+        }
+        else {
+          Msg::Error("Could not find GEO curve %d", (*ite)->tag());
         }
       }
       Tree_Add(_geo_internals->Surfaces, &s);
-      List_Delete(curves);
     }
   }
 
-  // TODO: create Volumes from discreteRegions ; meanwhile, keep track of
-  // maximum volume tag so that we don't break later operations:
-  _geo_internals->setMaxTag(3, maxv);
+  for(riter it = firstRegion(); it != lastRegion(); it++) {
+    if((*it)->geomType() == GEntity::DiscreteVolume) {
+      Volume *v = CreateVolume((*it)->tag(), MSH_VOLUME_DISCRETE);
+      std::vector<GFace *> faces = (*it)->faces();
+      v->Surfaces = List_Create(faces.size() + 1, 1, sizeof(Surface *));
+      for(std::vector<GFace *>::iterator itf = faces.begin();
+          itf != faces.end(); itf++) {
+        Surface *s = FindSurface((*itf)->tag());
+        if(s) {
+          List_Add(v->Surfaces, &s);
+        }
+        else {
+          Msg::Error("Could not find GEO surface %d", (*itf)->tag());
+        }
+      }
+      Tree_Add(_geo_internals->Volumes, &v);
+    }
+  }
 
   Msg::Debug("Geo internal model has:");
   Msg::Debug("%d points", Tree_Nbr(_geo_internals->Points));
   Msg::Debug("%d curves", Tree_Nbr(_geo_internals->Curves));
   Msg::Debug("%d surfaces", Tree_Nbr(_geo_internals->Surfaces));
+  Msg::Debug("%d volumes", Tree_Nbr(_geo_internals->Volumes));
 
   return 1;
 }
