@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2019 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2020 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
@@ -16,6 +16,8 @@
 #include "GmshMessage.h"
 #include "VertexArray.h"
 #include "boundaryLayersData.h"
+#include "discreteEdge.h"
+#include "discreteFace.h"
 #include "ExtrudeParams.h"
 #include "GmshDefines.h"
 
@@ -174,7 +176,7 @@ void GRegion::resetMeshAttributes()
   meshAttributes.QuadTri = NO_QUADTRI;
 }
 
-SBoundingBox3d GRegion::bounds(bool fast) const
+SBoundingBox3d GRegion::bounds(bool fast)
 {
   SBoundingBox3d res;
   if(geomType() != DiscreteVolume && geomType() != PartitionVolume) {
@@ -300,6 +302,43 @@ int GRegion::delFace(GFace *face)
   return orientation;
 }
 
+void GRegion::setBoundFaces(const std::set<int> &tagFaces)
+{
+  for(std::set<int>::const_iterator it = tagFaces.begin(); it != tagFaces.end();
+      ++it) {
+    GFace *face = model()->getFaceByTag(*it);
+    if(face) {
+      l_faces.push_back(face);
+      face->addRegion(this);
+    }
+    else {
+      Msg::Error("Unknown surface %d in volume %d", *it, tag());
+    }
+  }
+}
+
+void GRegion::setBoundFaces(const std::vector<int> &tagFaces,
+                            const std::vector<int> &signFaces)
+{
+  if(tagFaces.size() != signFaces.size()) {
+    Msg::Error("Wrong number of surface signs in volume %d", tag());
+    std::set<int> tags;
+    tags.insert(tagFaces.begin(), tagFaces.end());
+    setBoundFaces(tags);
+  }
+  for(std::size_t i = 0; i != tagFaces.size(); i++) {
+    GFace *face = model()->getFaceByTag(tagFaces[i]);
+    if(face) {
+      l_faces.push_back(face);
+      face->addRegion(this);
+      l_dirs.push_back(signFaces[i]);
+    }
+    else {
+      Msg::Error("Unknown surface %d in volume %d", tagFaces[i], tag());
+    }
+  }
+}
+
 std::string GRegion::getAdditionalInfoString(bool multline)
 {
   std::ostringstream sstream;
@@ -529,6 +568,41 @@ double GRegion::computeSolidProperties(std::vector<double> cg,
   return volume;
 }
 
+std::vector<MVertex *> GRegion::getEmbeddedMeshVertices() const
+{
+  std::set<MVertex *> tmp;
+  for(std::vector<GFace *>::const_iterator it = embedded_faces.begin();
+      it != embedded_faces.end(); it++) {
+    tmp.insert((*it)->mesh_vertices.begin(), (*it)->mesh_vertices.end());
+    std::vector<GEdge*> ed = (*it)->edges();
+    for(std::vector<GEdge *>::const_iterator it2 = ed.begin();
+        it2 != ed.end(); it2++) {
+      tmp.insert((*it2)->mesh_vertices.begin(), (*it2)->mesh_vertices.end());
+      if((*it2)->getBeginVertex())
+        tmp.insert((*it2)->getBeginVertex()->mesh_vertices.begin(),
+                   (*it2)->getBeginVertex()->mesh_vertices.end());
+      if((*it2)->getEndVertex())
+        tmp.insert((*it2)->getEndVertex()->mesh_vertices.begin(),
+                   (*it2)->getEndVertex()->mesh_vertices.end());
+    }
+  }
+  for(std::vector<GEdge *>::const_iterator it = embedded_edges.begin();
+      it != embedded_edges.end(); it++) {
+    tmp.insert((*it)->mesh_vertices.begin(), (*it)->mesh_vertices.end());
+    if((*it)->getBeginVertex())
+      tmp.insert((*it)->getBeginVertex()->mesh_vertices.begin(),
+                 (*it)->getBeginVertex()->mesh_vertices.end());
+    if((*it)->getEndVertex())
+      tmp.insert((*it)->getEndVertex()->mesh_vertices.begin(),
+                 (*it)->getEndVertex()->mesh_vertices.end());
+  }
+  for(std::vector<GVertex *>::const_iterator it = embedded_vertices.begin();
+      it != embedded_vertices.end(); it++) {
+    tmp.insert((*it)->mesh_vertices.begin(), (*it)->mesh_vertices.end());
+  }
+  return std::vector<MVertex *>(tmp.begin(), tmp.end());
+}
+
 std::vector<GVertex *> GRegion::vertices() const
 {
   std::set<GVertex *> v;
@@ -889,3 +963,22 @@ bool GRegion::setOutwardOrientationMeshConstraint()
 
   return true;
 }
+
+bool GRegion::isFullyDiscrete()
+{
+  if(geomType() != GEntity::DiscreteVolume) return false;
+  if(haveParametrization()) return false;
+  std::vector<GFace *> f = faces();
+  for(std::size_t i = 0; i < f.size(); i++) {
+    if(f[i]->geomType() != GEntity::DiscreteSurface) return false;
+    discreteFace *df = dynamic_cast<discreteFace*>(f[i]);
+    if(df && df->haveParametrization()) return false;
+  }
+  std::vector<GEdge *> e = edges();
+  for(std::size_t i = 0; i < e.size(); i++) {
+    if(e[i]->geomType() != GEntity::DiscreteCurve) return false;
+    discreteEdge *de = dynamic_cast<discreteEdge*>(e[i]);
+    if(de && de->haveParametrization()) return false;
+  }
+  return true;
+}
