@@ -27,7 +27,6 @@
 #include "meshGRegionLocalMeshMod.h"
 #include "meshRelocateVertex.h"
 #include "meshRefine.h"
-#include "discreteFace.h"
 #include "BackgroundMesh.h"
 #include "BoundaryLayers.h"
 #include "ExtrudeParams.h"
@@ -337,11 +336,13 @@ static void Mesh0D(GModel *m)
 
 static void Mesh1D(GModel *m)
 {
+  if(CTX::instance()->abortOnError && Msg::GetErrorCount()) return;
+
   m->getFields()->initialize();
 
   if(TooManyElements(m, 1)) return;
   Msg::StatusBar(true, "Meshing 1D...");
-  double t1 = Cpu();
+  double t1 = Cpu(), w1 = TimeOfDay();
 
   int prevNumThreads = Msg::GetMaxThreads();
   if(CTX::instance()->mesh.maxNumThreads1D > 0 &&
@@ -368,6 +369,11 @@ static void Mesh1D(GModel *m)
   Msg::StartProgressMeter(nTot);
 
   while(1) {
+    if(CTX::instance()->abortOnError && Msg::GetErrorCount()) {
+      Msg::Error("Aborted 1D meshing");
+      break;
+    }
+
     int nPending = 0;
     const size_t sss = temp.size();
 #if defined(_OPENMP)
@@ -395,9 +401,10 @@ static void Mesh1D(GModel *m)
 
   Msg::SetNumThreads(prevNumThreads);
 
-  double t2 = Cpu();
-  CTX::instance()->meshTimer[0] = t2 - t1;
-  Msg::StatusBar(true, "Done meshing 1D (%g s)", CTX::instance()->meshTimer[0]);
+  double t2 = Cpu(), w2 = TimeOfDay();
+  CTX::instance()->meshTimer[0] = w2 - w1;
+  Msg::StatusBar(true, "Done meshing 1D (Wall %gs, CPU %gs)",
+                 CTX::instance()->meshTimer[0], t2 - t1);
 }
 
 static void PrintMesh2dStatistics(GModel *m)
@@ -467,11 +474,13 @@ static void PrintMesh2dStatistics(GModel *m)
 
 static void Mesh2D(GModel *m)
 {
+  if(CTX::instance()->abortOnError && Msg::GetErrorCount()) return;
+
   m->getFields()->initialize();
 
   if(TooManyElements(m, 2)) return;
   Msg::StatusBar(true, "Meshing 2D...");
-  double t1 = Cpu();
+  double t1 = Cpu(), w1 = TimeOfDay();
 
   int prevNumThreads = Msg::GetMaxThreads();
   if(CTX::instance()->mesh.maxNumThreads2D > 0 &&
@@ -482,11 +491,6 @@ static void Mesh2D(GModel *m)
   if(m->getFields()->getNumBoundaryLayerFields()) Msg::SetNumThreads(1);
 
   for(GModel::fiter it = m->firstFace(); it != m->lastFace(); ++it) {
-    // STL remeshing is not yet thread-safe
-    //    if((*it)->geomType() == GEntity::DiscreteSurface){
-    //      if(static_cast<discreteFace *>(*it)->haveParametrization())
-    //        Msg::SetNumThreads(1);
-    //    }
     // Frontal-Delaunay for quads and co are not yet thread-safe
     if((*it)->getMeshingAlgo() == ALGO_2D_FRONTAL_QUAD ||
        (*it)->getMeshingAlgo() == ALGO_2D_PACK_PRLGRMS ||
@@ -516,6 +520,11 @@ static void Mesh2D(GModel *m)
     Msg::StartProgressMeter(nTot);
 
     while(1) {
+      if(CTX::instance()->abortOnError && Msg::GetErrorCount()) {
+        Msg::Error("Aborted 2D meshing");
+        break;
+      }
+
       int nPending = 0;
       std::vector<GFace *> temp;
       temp.insert(temp.begin(), f.begin(), f.end());
@@ -547,9 +556,10 @@ static void Mesh2D(GModel *m)
 
   Msg::SetNumThreads(prevNumThreads);
 
-  double t2 = Cpu();
-  CTX::instance()->meshTimer[1] = t2 - t1;
-  Msg::StatusBar(true, "Done meshing 2D (%g s)", CTX::instance()->meshTimer[1]);
+  double t2 = Cpu(), w2 = TimeOfDay();
+  CTX::instance()->meshTimer[1] = w2 - w1;
+  Msg::StatusBar(true, "Done meshing 2D (Wall %gs, CPU %gs)",
+                 CTX::instance()->meshTimer[1], t2 - t1);
 
   PrintMesh2dStatistics(m);
 }
@@ -799,11 +809,13 @@ static void TestConformity(GModel *gm)
 
 static void Mesh3D(GModel *m)
 {
+  if(CTX::instance()->abortOnError && Msg::GetErrorCount()) return;
+
   m->getFields()->initialize();
 
   if(TooManyElements(m, 3)) return;
   Msg::StatusBar(true, "Meshing 3D...");
-  double t1 = Cpu();
+  double t1 = Cpu(), w1 = TimeOfDay();
 
   int prevNumThreads = Msg::GetMaxThreads();
   if(CTX::instance()->mesh.maxNumThreads3D > 0 &&
@@ -859,6 +871,11 @@ static void Mesh3D(GModel *m)
 #endif
 
   for(std::size_t i = 0; i < connected.size(); i++) {
+    if(CTX::instance()->abortOnError && Msg::GetErrorCount()) {
+      Msg::Error("Aborted 3D meshing");
+      break;
+    }
+
     MeshDelaunayVolume(connected[i]);
 
 #if defined(HAVE_DOMHEX)
@@ -884,7 +901,7 @@ static void Mesh3D(GModel *m)
           optimizeMeshGRegion opt;
           opt(gr);
         }
-        double a = Cpu();
+        double a = TimeOfDay();
         // CTX::instance()->mesh.recombine3DLevel = 2;
         if(CTX::instance()->mesh.recombine3DLevel >= 0) {
           Recombinator rec;
@@ -907,7 +924,7 @@ static void Mesh3D(GModel *m)
         nb_hexa_recombination += post.get_nb_hexahedra();
         vol_element_recombination += post.get_vol_elements();
         vol_hexa_recombination += post.get_vol_hexahedra();
-        time_recombination += (Cpu() - a);
+        time_recombination += (TimeOfDay() - a);
       }
     }
 #endif
@@ -939,7 +956,10 @@ static void Mesh3D(GModel *m)
   bool emptyRegionFound = false;
   for(GModel::riter it = m->firstRegion(); it != m->lastRegion(); ++it) {
     GRegion *gr = *it;
-    if(CTX::instance()->mesh.meshOnlyVisible && !gr->getVisibility()) continue;
+    if(CTX::instance()->mesh.meshOnlyVisible && !gr->getVisibility())
+      continue;
+    if(CTX::instance()->mesh.meshOnlyEmpty && gr->getNumMeshElements())
+      continue;
     if(gr->getNumMeshElements() == 0) {
       debugInfo << gr->tag() << " ";
       emptyRegionFound = true;
@@ -952,19 +972,22 @@ static void Mesh3D(GModel *m)
 
   Msg::SetNumThreads(prevNumThreads);
 
-  double t2 = Cpu();
-  CTX::instance()->meshTimer[2] = t2 - t1;
+  double t2 = Cpu(), w2 = TimeOfDay();
+  CTX::instance()->meshTimer[2] = w2 - w1;
 
   if(m->getNumRegions()) {
     Msg::ProgressMeter(1, false, "Meshing 3D...");
     Msg::StopProgressMeter();
   }
 
-  Msg::StatusBar(true, "Done meshing 3D (%g s)", CTX::instance()->meshTimer[2]);
+  Msg::StatusBar(true, "Done meshing 3D (Wall %gs, CPU %gs)",
+                 CTX::instance()->meshTimer[2], t2 - t1);
 }
 
 void OptimizeMesh(GModel *m, const std::string &how, bool force, int niter)
 {
+  if(CTX::instance()->abortOnError && Msg::GetErrorCount()) return;
+
   if(how != "" && how != "Gmsh" && how != "Optimize" && how != "Netgen" &&
      how != "HighOrder" && how != "HighOrderElastic" &&
      how != "HighOrderFastCurving" && how != "Laplace2D" &&
@@ -977,7 +1000,7 @@ void OptimizeMesh(GModel *m, const std::string &how, bool force, int niter)
     Msg::StatusBar(true, "Optimizing mesh...");
   else
     Msg::StatusBar(true, "Optimizing mesh (%s)...", how.c_str());
-  double t1 = Cpu();
+  double t1 = Cpu(), w1 = TimeOfDay();
 
   if(how == "" || how == "Gmsh" || how == "Optimize") {
     for(GModel::riter it = m->firstRegion(); it != m->lastRegion(); it++) {
@@ -1050,26 +1073,32 @@ void OptimizeMesh(GModel *m, const std::string &how, bool force, int niter)
     std::for_each(m->firstRegion(), m->lastRegion(),
                   EmbeddedCompatibilityTest());
 
-  double t2 = Cpu();
-  Msg::StatusBar(true, "Done optimizing mesh (%g s)", t2 - t1);
+  double t2 = Cpu(), w2 = TimeOfDay();
+  Msg::StatusBar(true, "Done optimizing mesh (Wall %gs, CPU %gs)",
+                 w2 - w1, t2 - t1);
 }
 
 void AdaptMesh(GModel *m)
 {
+  if(CTX::instance()->abortOnError && Msg::GetErrorCount()) return;
+
   Msg::StatusBar(true, "Adapting 3D mesh...");
-  double t1 = Cpu();
+  double t1 = Cpu(), w1 = TimeOfDay();
 
   for(int i = 0; i < 10; i++)
     std::for_each(m->firstRegion(), m->lastRegion(), adaptMeshGRegion());
 
-  double t2 = Cpu();
-  Msg::StatusBar(true, "Done adaptating 3D mesh (%g s)", t2 - t1);
+  double t2 = Cpu(), w2 = TimeOfDay();
+  Msg::StatusBar(true, "Done adaptating 3D mesh (Wall %gs, CPU %gs)",
+                 w2 - w1, t2 - t1);
 }
 
 void RecombineMesh(GModel *m)
 {
+  if(CTX::instance()->abortOnError && Msg::GetErrorCount()) return;
+
   Msg::StatusBar(true, "Recombining 2D mesh...");
-  double t1 = Cpu();
+  double t1 = Cpu(), w1 = TimeOfDay();
 
   for(GModel::fiter it = m->firstFace(); it != m->lastFace(); ++it) {
     GFace *gf = *it;
@@ -1079,8 +1108,9 @@ void RecombineMesh(GModel *m)
     recombineIntoQuads(gf, blossom, topo, true, .01);
   }
 
-  double t2 = Cpu();
-  Msg::StatusBar(true, "Done recombining 2D mesh (%g s)", t2 - t1);
+  double t2 = Cpu(), w2 = TimeOfDay();
+  Msg::StatusBar(true, "Done recombining 2D mesh (Wall %gs, CPU %gs)",
+                 w2 - w1, t2 - t1);
 }
 
 static SPoint3 transform(MVertex *vsource, const std::vector<double> &tfo)
@@ -1170,7 +1200,7 @@ static void relocateSlaveVertices(std::vector<GEntity *> &entities,
                 useClosestPoint ? " (using closest point)" : "");
       relocateSlaveVertices(slave, slave->correspondingVertices,
                             useClosestPoint);
-      relocateSlaveVertices(slave, slave->correspondingHOPoints,
+      relocateSlaveVertices(slave, slave->correspondingHighOrderVertices,
                             useClosestPoint);
     }
     else if(it->first->dim() == 1) {
@@ -1182,7 +1212,7 @@ static void relocateSlaveVertices(std::vector<GEntity *> &entities,
                 useClosestPoint ? " (using closest point)" : "");
       relocateSlaveVertices(slave, slave->correspondingVertices,
                             useClosestPoint);
-      relocateSlaveVertices(slave, slave->correspondingHOPoints,
+      relocateSlaveVertices(slave, slave->correspondingHighOrderVertices,
                             useClosestPoint);
     }
   }
@@ -1190,6 +1220,8 @@ static void relocateSlaveVertices(std::vector<GEntity *> &entities,
 
 void FixPeriodicMesh(GModel *m)
 {
+  if(CTX::instance()->abortOnError && Msg::GetErrorCount()) return;
+
   for(GModel::eiter it = m->firstEdge(); it != m->lastEdge(); ++it) {
     GEdge *tgt = *it;
 
@@ -1200,7 +1232,7 @@ void FixPeriodicMesh(GModel *m)
 
     if(src != NULL && src != tgt) {
       std::map<MVertex *, MVertex *> &v2v = tgt->correspondingVertices;
-      std::map<MVertex *, MVertex *> &p2p = tgt->correspondingHOPoints;
+      std::map<MVertex *, MVertex *> &p2p = tgt->correspondingHighOrderVertices;
       p2p.clear();
 
       Msg::Info("Reconstructing periodicity for curve connection %d - %d",
@@ -1274,7 +1306,7 @@ void FixPeriodicMesh(GModel *m)
                 tgt->tag(), src->tag());
 
       std::map<MVertex *, MVertex *> &v2v = tgt->correspondingVertices;
-      std::map<MVertex *, MVertex *> &p2p = tgt->correspondingHOPoints;
+      std::map<MVertex *, MVertex *> &p2p = tgt->correspondingHighOrderVertices;
       p2p.clear();
 
       if(tgt->getNumMeshElements() && v2v.empty()) {
@@ -1374,8 +1406,9 @@ void GenerateMesh(GModel *m, int ask)
   // Initialize pseudo random mesh generator with the same seed
   srand(CTX::instance()->mesh.randomSeed);
 
-  // Change any high order elements back into first order ones
-  SetOrder1(m);
+  // Change any high order elements back into first order ones (but skip
+  // discrete entities)
+  SetOrder1(m, false, true);
   FixPeriodicMesh(m);
 
   // 1D mesh
@@ -1403,7 +1436,9 @@ void GenerateMesh(GModel *m, int ask)
     std::for_each(m->firstFace(), m->lastFace(), orientMeshGFace());
 
   // Optimize quality of 3D tet mesh
-  if(m->getMeshStatus() == 3 && CTX::instance()->mesh.algo3d != ALGO_3D_HXT) {
+  if(m->getMeshStatus() == 3 &&
+     CTX::instance()->mesh.algo3d != ALGO_3D_INITIAL_ONLY &&
+     CTX::instance()->mesh.algo3d != ALGO_3D_HXT) {
     for(int i = 0; i < std::max(CTX::instance()->mesh.optimize,
                                 CTX::instance()->mesh.optimizeNetgen);
         i++) {
