@@ -344,15 +344,9 @@ static void copyMesh(GEdge *from, GEdge *to, int direction)
 
 void deMeshGEdge::operator()(GEdge *ge)
 {
-  if(ge->geomType() == GEntity::DiscreteCurve){
-    if(!static_cast<discreteEdge *>(ge)->haveParametrization()){
-      return;
-    }
-  }
+  if(ge->isFullyDiscrete()) return;
   ge->deleteMesh();
   ge->meshStatistics.status = GEdge::PENDING;
-  ge->correspondingVertices.clear();
-  ge->correspondingHOPoints.clear();
 }
 
 /*
@@ -463,8 +457,8 @@ static void createPoints(GVertex *gv, GEdge *ge, BoundaryLayerField *blf,
 {
   if(!ge->getBeginVertex() || !ge->getEndVertex()) return;
 
-  const double hwall = blf->hwall(gv->tag());
-  double L = hwall;
+  const double hWall = blf->hWall(gv->tag());
+  double L = hWall;
   double LEdge = distance(ge->getBeginVertex()->mesh_vertices[0],
                           ge->getEndVertex()->mesh_vertices[0]);
   while(1) {
@@ -474,9 +468,9 @@ static void createPoints(GVertex *gv, GEdge *ge, BoundaryLayerField *blf,
 
     SPoint3 p(gv->x() + dir.x() * L, gv->y() + dir.y() * L, 0.0);
     v.push_back(new MEdgeVertex(p.x(), p.y(), p.z(), ge, ge->parFromPoint(p), 0,
-                                blf->hfar));
+                                blf->hFar));
     int ith = v.size();
-    L += hwall * std::pow(blf->ratio, ith);
+    L += hWall * std::pow(blf->ratio, ith);
   }
 }
 
@@ -569,12 +563,12 @@ void meshGEdge::operator()(GEdge *ge)
     return;
   }
 
-  if ((ge->geomType() == GEntity::BoundaryLayerCurve) ||
-     (ge->meshAttributes.method == MESH_NONE) ||
-     (CTX::instance()->mesh.meshOnlyVisible && !ge->getVisibility())) {
-    return;
-  }
+  if(ge->geomType() == GEntity::BoundaryLayerCurve) return;
+  if(ge->meshAttributes.method == MESH_NONE) return;
+  if(CTX::instance()->mesh.meshOnlyVisible && !ge->getVisibility()) return;
+  if(CTX::instance()->mesh.meshOnlyEmpty && ge->getNumMeshElements()) return;
 
+  // destroy the mesh if it exists
   deMeshGEdge dem;
   dem(ge);
 
