@@ -20,15 +20,10 @@ class MVertex;
 // Create the mesh of the face
 class meshGFace {
   const bool repairSelfIntersecting1dMesh;
-  bool onlyInitialMesh;
 
 public:
-  meshGFace(bool r = true)
-    : repairSelfIntersecting1dMesh(r), onlyInitialMesh(false)
-  {
-  }
+  meshGFace(bool r = true) : repairSelfIntersecting1dMesh(r) {}
   void operator()(GFace *, bool print = true);
-  void setOnlyInitial() { onlyInitialMesh = true; }
 };
 
 // Destroy the mesh of the face
