@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2019 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2020 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
@@ -93,30 +93,35 @@ extrudeMesh(GEdge *from, GFace *to, MVertexRTree &pos,
     for(std::size_t i = 0; i < from->mesh_vertices.size(); i++) {
       std::vector<MVertex *> extruded_vertices;
       MVertex *v = from->mesh_vertices[i];
-      MEdgeVertex *mv = (MEdgeVertex *)v;
-      mv->bl_data = new MVertexBoundaryLayerData();
+      MEdgeVertex *mv = dynamic_cast<MEdgeVertex *>(v);
+      if(mv) mv->bl_data = new MVertexBoundaryLayerData();
       for(int j = 0; j < ep->mesh.NbLayer; j++) {
         for(int k = 0; k < ep->mesh.NbElmLayer[j]; k++) {
           double x = v->x(), y = v->y(), z = v->z();
           ep->Extrude(j, k + 1, x, y, z);
           if(j != ep->mesh.NbLayer - 1 || k != ep->mesh.NbElmLayer[j] - 1) {
-            MVertex *newv = 0;
-            if(to->geomType() != GEntity::DiscreteSurface &&
-               to->geomType() != GEntity::BoundaryLayerSurface) {
-              SPoint3 xyz(x, y, z);
-              SPoint2 uv = to->parFromPoint(xyz);
-              newv = new MFaceVertex(x, y, z, to, uv[0], uv[1]);
+            MVertex *newv = pos.find(x, y, z);
+            if(!newv) {
+              if(from->geomType() != GEntity::DiscreteCurve &&
+                 to->geomType() != GEntity::DiscreteSurface &&
+                 to->geomType() != GEntity::BoundaryLayerSurface) {
+                // This can be inefficient, and sometimes useless. We could add an
+                // option to disable it.
+                SPoint3 xyz(x, y, z);
+                SPoint2 uv = to->parFromPoint(xyz);
+                newv = new MFaceVertex(x, y, z, to, uv[0], uv[1]);
+              }
+              else {
+                newv = new MVertex(x, y, z, to);
+              }
+              to->mesh_vertices.push_back(newv);
             }
-            else {
-              newv = new MVertex(x, y, z, to);
-            }
-            to->mesh_vertices.push_back(newv);
             pos.insert(newv);
             extruded_vertices.push_back(newv);
           }
         }
       }
-      mv->bl_data->addChildrenFamily(extruded_vertices);
+      if(mv) mv->bl_data->addChildrenFamily(extruded_vertices);
     }
   }
 
@@ -157,7 +162,7 @@ extrudeMesh(GEdge *from, GFace *to, MVertexRTree &pos,
         for(int p = 0; p < 4; p++) {
           MVertex *tmp = pos.find(x[p], y[p], z[p]);
           if(!tmp) {
-            Msg::Error("Could not find extruded vertex (%.16g, %.16g, %.16g) "
+            Msg::Error("Could not find extruded node (%.16g, %.16g, %.16g) "
                        "in surface %d",
                        x[p], y[p], z[p], to->tag());
             return;
@@ -188,25 +193,22 @@ static void copyMesh(GFace *from, GFace *to, MVertexRTree &pos)
     double x = v->x(), y = v->y(), z = v->z();
     ep->Extrude(ep->mesh.NbLayer - 1, ep->mesh.NbElmLayer[ep->mesh.NbLayer - 1],
                 x, y, z);
-    MVertex *newv = 0;
-    if(to->geomType() != GEntity::DiscreteSurface &&
-       to->geomType() != GEntity::BoundaryLayerSurface) {
-      SPoint3 xyz(x, y, z);
-      SPoint2 uv = to->parFromPoint(xyz);
-      newv = new MFaceVertex(x, y, z, to, uv[0], uv[1]);
+    MVertex *newv = pos.find(x, y, z);
+    if(!newv) {
+      if(to->geomType() != GEntity::DiscreteSurface &&
+         to->geomType() != GEntity::BoundaryLayerSurface) {
+        SPoint3 xyz(x, y, z);
+        SPoint2 uv = to->parFromPoint(xyz);
+        newv = new MFaceVertex(x, y, z, to, uv[0], uv[1]);
+      }
+      else {
+        newv = new MVertex(x, y, z, to);
+      }
+      to->mesh_vertices.push_back(newv);
+      pos.insert(newv);
     }
-    else {
-      newv = new MVertex(x, y, z, to);
-    }
-    to->mesh_vertices.push_back(newv);
-    to->correspondingVertices[newv] = v;
-    pos.insert(newv);
   }
 
-  std::vector<double> tfo;
-  //ep->GetAffineTransform(tfo); // TODO: check transform
-  to->GEntity::setMeshMaster(from, tfo, false);
-
 #if defined(HAVE_QUADTRI)
   // if performing QuadToTri mesh, cannot simply copy the mesh from the source.
   // The vertices and triangles can be copied directly though.  First, of
@@ -250,7 +252,7 @@ static void copyMesh(GFace *from, GFace *to, MVertexRTree &pos)
       MVertex *tmp = pos.find(x, y, z);
       if(!tmp) {
         Msg::Error(
-          "Could not find extruded vertex (%.16g, %.16g, %.16g) in surface %d",
+          "Could not find extruded node (%.16g, %.16g, %.16g) in surface %d",
           x, y, z, to->tag());
         return;
       }
@@ -282,7 +284,7 @@ static void copyMesh(GFace *from, GFace *to, MVertexRTree &pos)
       MVertex *tmp = pos.find(x, y, z);
       if(!tmp) {
         Msg::Error(
-          "Could not find extruded vertex (%.16g, %.16g, %.16g) in surface %d",
+          "Could not find extruded node (%.16g, %.16g, %.16g) in surface %d",
           x, y, z, to->tag());
         return;
       }
@@ -299,26 +301,22 @@ int MeshExtrudedSurface(
 
   if(!ep || !ep->mesh.ExtrudeMesh) return 0;
 
-  Msg::Info("Meshing surface %d (extruded)", gf->tag());
+  Msg::Info("Meshing surface %d (Extruded)", gf->tag());
 
-  // build an rtree with all the vertices on the boundary of the face gf
+  // build an rtree with all the vertices on the face and its boundary
   MVertexRTree pos(CTX::instance()->geom.tolerance * CTX::instance()->lc);
+  pos.insert(gf->mesh_vertices);
   std::vector<GEdge *> const &edges = gf->edges();
-  std::vector<GEdge *>::const_iterator it = edges.begin();
-  while(it != edges.end()) {
+  for(std::vector<GEdge *>::const_iterator it = edges.begin();
+      it != edges.end(); it++) {
     pos.insert((*it)->mesh_vertices);
     if((*it)->getBeginVertex())
       pos.insert((*it)->getBeginVertex()->mesh_vertices);
     if((*it)->getEndVertex())
       pos.insert((*it)->getEndVertex()->mesh_vertices);
-    ++it;
-  }
-
-  // if the edges of the mesh are constrained, the vertices already
-  // exist on the face--so we add them to the set
-  if(constrainedEdges) {
-    pos.insert(gf->mesh_vertices);
   }
+  std::vector<MVertex*> embedded = gf->getEmbeddedMeshVertices();
+  pos.insert(embedded);
 
   if(ep->geo.Mode == EXTRUDED_ENTITY) {
     // surface is extruded from a curve
@@ -343,6 +341,10 @@ int MeshExtrudedSurface(
       return 1;
     }
     copyMesh(from, gf, pos);
+    if(gf->getMeshMaster() == from) {
+      // explicit periodic constraint, to store node correspondance
+      gf->setMeshMaster(from, gf->affineTransform);
+    }
   }
 
   gf->meshStatistics.status = GFace::DONE;
