@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2019 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2020 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
@@ -20,6 +20,8 @@
 #include "GRegion.h"
 #include "GFace.h"
 #include "GEdge.h"
+#include "discreteFace.h"
+#include "discreteEdge.h"
 #include "MLine.h"
 #include "MTriangle.h"
 #include "MTetrahedron.h"
@@ -50,7 +52,7 @@ int splitQuadRecovery::buildPyramids(GModel *gm)
   for(GModel::riter it = gm->firstRegion(); it != gm->lastRegion(); it++){
     GRegion *gr = *it;
     if(gr->meshAttributes.method == MESH_TRANSFINITE) continue;
-    if(gr->geomType() == GEntity::DiscreteVolume) continue;
+    if(gr->isFullyDiscrete()) continue;
     ExtrudeParams *ep = gr->meshAttributes.extrude;
     if(ep && ep->mesh.ExtrudeMesh && ep->geo.Mode == EXTRUDED_ENTITY) continue;
 
@@ -58,7 +60,7 @@ int splitQuadRecovery::buildPyramids(GModel *gm)
     for(std::size_t i = 0; i < faces.size(); i++){
       GFace *gf = faces[i];
       for(std::size_t j = 0; j < gf->quadrangles.size(); j++){
-        std::map<MFace, MVertex *, Less_Face>::iterator it2 =
+        std::map<MFace, MVertex *, MFaceLessThan>::iterator it2 =
           _quad.find(gf->quadrangles[j]->getFace(0));
         if(it2 != _quad.end()){
           npyram++;
@@ -95,13 +97,14 @@ void MeshDelaunayVolume(std::vector<GRegion *> &regions)
   }
 
   if(CTX::instance()->mesh.algo3d != ALGO_3D_DELAUNAY &&
+     CTX::instance()->mesh.algo3d != ALGO_3D_INITIAL_ONLY &&
      CTX::instance()->mesh.algo3d != ALGO_3D_MMG3D)
     return;
 
   GRegion *gr = regions[0];
   std::vector<GFace *> faces = gr->faces();
 
-  std::set<GFace *, GEntityLessThan> allFacesSet;
+  std::set<GFace *, GEntityPtrLessThan> allFacesSet;
   for(std::size_t i = 0; i < regions.size(); i++) {
     std::vector<GFace *> const &f = regions[i]->faces();
     std::vector<GFace *> const &f_e = regions[i]->embeddedFaces();
@@ -112,8 +115,8 @@ void MeshDelaunayVolume(std::vector<GRegion *> &regions)
   // replace faces with compounds if elements from compound surface meshes are
   // not reclassified on the original surfaces
   if(CTX::instance()->mesh.compoundClassify == 0){
-    std::set<GFace *, GEntityLessThan> comp;
-    for(std::set<GFace *, GEntityLessThan>::iterator it = allFacesSet.begin();
+    std::set<GFace *, GEntityPtrLessThan> comp;
+    for(std::set<GFace *, GEntityPtrLessThan>::iterator it = allFacesSet.begin();
         it != allFacesSet.end(); it++){
       GFace *gf = *it;
       if(!gf->compoundSurface)
@@ -127,7 +130,7 @@ void MeshDelaunayVolume(std::vector<GRegion *> &regions)
   std::vector<GFace *> allFaces(allFacesSet.begin(), allFacesSet.end());
   gr->set(allFaces);
 
-  std::set<GEdge *> allEmbEdgesSet;
+  std::set<GEdge *, GEntityPtrLessThan> allEmbEdgesSet;
   for(std::size_t i = 0; i < regions.size(); i++) {
     std::vector<GEdge *> const &e = regions[i]->embeddedEdges();
     allEmbEdgesSet.insert(e.begin(), e.end());
@@ -160,7 +163,7 @@ void MeshDelaunayVolume(std::vector<GRegion *> &regions)
 
   // restore set of faces and embedded edges/vertices
   if(CTX::instance()->mesh.compoundClassify == 0){
-    std::set<GFace *, GEntityLessThan> comp;
+    std::set<GFace *, GEntityPtrLessThan> comp;
     for(std::size_t i = 0; i < faces.size(); i++) {
       GFace *gf = faces[i];
       if(!gf->compoundSurface)
@@ -183,8 +186,9 @@ void MeshDelaunayVolume(std::vector<GRegion *> &regions)
   if(CTX::instance()->mesh.algo3d == ALGO_3D_MMG3D) {
     refineMeshMMG(gr);
   }
-  else{
-    insertVerticesInRegion(gr, 2000000000, true, &sqr);
+  else if(CTX::instance()->mesh.algo3d != ALGO_3D_INITIAL_ONLY) {
+    insertVerticesInRegion(gr, CTX::instance()->mesh.maxIterDelaunay3D,
+                           1., true, &sqr);
 
     if(sqr.buildPyramids(gr->model())){
       Msg::Info("Optimizing pyramids for hybrid mesh...");
@@ -203,7 +207,7 @@ void MeshDelaunayVolume(std::vector<GRegion *> &regions)
 
 void deMeshGRegion::operator()(GRegion *gr)
 {
-  if(gr->geomType() == GEntity::DiscreteVolume) return;
+  if(gr->isFullyDiscrete()) return;
   gr->deleteMesh();
 }
 
@@ -211,9 +215,10 @@ void meshGRegion::operator()(GRegion *gr)
 {
   gr->model()->setCurrentMeshEntity(gr);
 
-  if(gr->geomType() == GEntity::DiscreteVolume) return;
+  if(gr->isFullyDiscrete()) return;
   if(gr->meshAttributes.method == MESH_NONE) return;
   if(CTX::instance()->mesh.meshOnlyVisible && !gr->getVisibility()) return;
+  if(CTX::instance()->mesh.meshOnlyEmpty && gr->getNumMeshElements()) return;
 
   ExtrudeParams *ep = gr->meshAttributes.extrude;
   if(ep && ep->mesh.ExtrudeMesh) return;
@@ -236,7 +241,7 @@ void optimizeMeshGRegion::operator()(GRegion *gr, bool always)
 {
   gr->model()->setCurrentMeshEntity(gr);
 
-  if(!always && gr->geomType() == GEntity::DiscreteVolume) return;
+  if(!always && gr->isFullyDiscrete()) return;
 
   // don't optimize extruded meshes
   if(gr->meshAttributes.method == MESH_TRANSFINITE) return;
