@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2019 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2020 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
@@ -32,13 +32,13 @@ static void testIfBoundaryIsRecovered(GRegion *gr)
   std::vector<GEdge *> const &e = gr->edges();
   std::vector<GFace *> f = gr->faces();
 
-  std::map<MEdge, GEdge *, Less_Edge> edges;
-  std::map<MFace, GFace *, Less_Face> faces;
+  std::map<MEdge, GEdge *, MEdgeLessThan> edges;
+  std::map<MFace, GFace *, MFaceLessThan> faces;
 
   std::vector<GEdge *>::const_iterator it = e.begin();
   std::vector<GFace *>::iterator itf = f.begin();
   for(; it != e.end(); ++it) {
-    for(unsigned int i = 0; i < (*it)->lines.size(); ++i) {
+    for(std::size_t i = 0; i < (*it)->lines.size(); ++i) {
       if(distance((*it)->lines[i]->getVertex(0),
                   (*it)->lines[i]->getVertex(1)) > 1.e-12)
         edges.insert(std::make_pair(
@@ -47,7 +47,7 @@ static void testIfBoundaryIsRecovered(GRegion *gr)
     }
   }
   for(; itf != f.end(); ++itf) {
-    for(unsigned int i = 0; i < (*itf)->triangles.size(); ++i) {
+    for(std::size_t i = 0; i < (*itf)->triangles.size(); ++i) {
       faces.insert(std::make_pair(MFace((*itf)->triangles[i]->getVertex(0),
                                         (*itf)->triangles[i]->getVertex(1),
                                         (*itf)->triangles[i]->getVertex(2)),
@@ -57,7 +57,7 @@ static void testIfBoundaryIsRecovered(GRegion *gr)
   Msg::Info("Searching for %d mesh edges and %d mesh faces among %d elements "
             "in region %d",
             edges.size(), faces.size(), gr->getNumMeshElements(), gr->tag());
-  for(unsigned int k = 0; k < gr->getNumMeshElements(); k++) {
+  for(std::size_t k = 0; k < gr->getNumMeshElements(); k++) {
     for(int j = 0; j < gr->getMeshElement(k)->getNumEdges(); j++) {
       edges.erase(gr->getMeshElement(k)->getEdge(j));
     }
@@ -79,7 +79,7 @@ struct edgeContainerB {
   std::vector<std::vector<MEdge> > _hash;
   std::size_t _size, _size_obj;
 
-  edgeContainerB(unsigned int N = 1000000)
+  edgeContainerB(std::size_t N = 1000000)
     : _hash(N), _size(0), _size_obj(sizeof(MEdge))
   {
   }
@@ -112,12 +112,12 @@ struct edgeContainerB {
 };
 
 static void createAllEmbeddedEdges(GRegion *gr,
-                                   std::set<MEdge, Less_Edge> &allEmbeddedEdges)
+                                   std::set<MEdge, MEdgeLessThan> &allEmbeddedEdges)
 {
   std::vector<GEdge *> const &e = gr->embeddedEdges();
   for(std::vector<GEdge *>::const_iterator it = e.begin(); it != e.end();
       ++it) {
-    for(unsigned int i = 0; i < (*it)->lines.size(); i++) {
+    for(std::size_t i = 0; i < (*it)->lines.size(); i++) {
       allEmbeddedEdges.insert(
         MEdge((*it)->lines[i]->getVertex(0), (*it)->lines[i]->getVertex(1)));
     }
@@ -129,7 +129,7 @@ static void createAllEmbeddedEdges(GRegion *gr, edgeContainerB &embedded)
   std::vector<GEdge *> const &e = gr->embeddedEdges();
   for(std::vector<GEdge *>::const_iterator it = e.begin(); it != e.end();
       ++it) {
-    for(unsigned int i = 0; i < (*it)->lines.size(); i++) {
+    for(std::size_t i = 0; i < (*it)->lines.size(); i++) {
       embedded.addNewEdge(
         MEdge((*it)->lines[i]->getVertex(0), (*it)->lines[i]->getVertex(1)));
     }
@@ -137,12 +137,12 @@ static void createAllEmbeddedEdges(GRegion *gr, edgeContainerB &embedded)
 }
 
 static void createAllEmbeddedFaces(GRegion *gr,
-                                   std::set<MFace, Less_Face> &allEmbeddedFaces)
+                                   std::set<MFace, MFaceLessThan> &allEmbeddedFaces)
 {
   std::vector<GFace *> const &f = gr->embeddedFaces();
   for(std::vector<GFace *>::const_iterator it = f.begin(); it != f.end();
       ++it) {
-    for(unsigned int i = 0; i < (*it)->triangles.size(); i++) {
+    for(std::size_t i = 0; i < (*it)->triangles.size(); i++) {
       allEmbeddedFaces.insert((*it)->triangles[i]->getFace(0));
     }
   }
@@ -236,7 +236,7 @@ void connectTets_vector2_templ(std::size_t _size, ITER beg, ITER end,
 
   std::sort(conn.begin(), conn.end());
 
-  for(unsigned int i = 0; i < conn.size() - 1; i++) {
+  for(std::size_t i = 0; i < conn.size() - 1; i++) {
     faceXtet &f1 = conn[i];
     faceXtet &f2 = conn[i + 1];
     if(f1 == f2 && f1.t1 != f2.t1) {
@@ -249,7 +249,7 @@ void connectTets_vector2_templ(std::size_t _size, ITER beg, ITER end,
 
 template <class ITER>
 void connectTets(ITER beg, ITER end,
-                 const std::set<MFace, Less_Face> *allEmbeddedFaces = 0)
+                 const std::set<MFace, MFaceLessThan> *allEmbeddedFaces = 0)
 {
   std::set<faceXtet> conn;
   while(beg != end) {
@@ -275,13 +275,13 @@ void connectTets(ITER beg, ITER end,
 }
 
 void connectTets(std::list<MTet4 *> &l,
-                 const std::set<MFace, Less_Face> *embeddedFaces)
+                 const std::set<MFace, MFaceLessThan> *embeddedFaces)
 {
   connectTets(l.begin(), l.end(), embeddedFaces);
 }
 
 void connectTets(std::vector<MTet4 *> &l,
-                 const std::set<MFace, Less_Face> *embeddedFaces)
+                 const std::set<MFace, MFaceLessThan> *embeddedFaces)
 {
   connectTets(l.begin(), l.end(), embeddedFaces);
 }
@@ -464,7 +464,7 @@ bool insertVertexB(std::vector<faceXtet> &shell, std::vector<MTet4 *> &cavity,
                    std::vector<double> &vSizes, std::vector<double> &vSizesBGM,
                    MTet4 *t, MTet4Factory &myFactory,
                    std::set<MTet4 *, compareTet4Ptr> &allTets,
-                   const std::set<MFace, Less_Face> &allEmbeddedFaces)
+                   const std::set<MFace, MFaceLessThan> &allEmbeddedFaces)
 {
   std::vector<MTet4 *> new_cavity;
   new_cavity.reserve(shell.size());
@@ -508,7 +508,7 @@ bool insertVertexB(std::vector<faceXtet> &shell, std::vector<MTet4 *> &cavity,
     return true;
   }
   else /* one point is too close */ {
-    for(unsigned int i = 0; i < shell.size(); i++) myFactory.Free(new_tets[i]);
+    for(std::size_t i = 0; i < shell.size(); i++) myFactory.Free(new_tets[i]);
     std::vector<MTet4 *>::iterator ittet = cavity.begin();
     std::vector<MTet4 *>::iterator ittete = cavity.end();
     while(ittet != ittete) {
@@ -520,8 +520,8 @@ bool insertVertexB(std::vector<faceXtet> &shell, std::vector<MTet4 *> &cavity,
 }
 
 static void setLcs(MTriangle *t,
-                   std::map<MVertex *, double, MVertexLessThanNum> &vSizes,
-                   std::set<MVertex *, MVertexLessThanNum> &bndVertices)
+                   std::map<MVertex *, double, MVertexPtrLessThan> &vSizes,
+                   std::set<MVertex *, MVertexPtrLessThan> &bndVertices)
 {
   for(int i = 0; i < 3; i++) {
     bndVertices.insert(t->getVertex(i));
@@ -532,9 +532,9 @@ static void setLcs(MTriangle *t,
     double dy = vi->y() - vj->y();
     double dz = vi->z() - vj->z();
     double l = std::sqrt(dx * dx + dy * dy + dz * dz);
-    std::map<MVertex *, double, MVertexLessThanNum>::iterator iti =
+    std::map<MVertex *, double, MVertexPtrLessThan>::iterator iti =
       vSizes.find(vi);
-    std::map<MVertex *, double, MVertexLessThanNum>::iterator itj =
+    std::map<MVertex *, double, MVertexPtrLessThan>::iterator itj =
       vSizes.find(vj);
     if(CTX::instance()->mesh.lcExtendFromBoundary == 2) {
       // use smallest edge length
@@ -576,8 +576,8 @@ static void setLcs(MTriangle *t,
 }
 
 static void setLcs(MTetrahedron *t,
-                   std::map<MVertex *, double, MVertexLessThanNum> &vSizes,
-                   std::set<MVertex *, MVertexLessThanNum> &bndVertices)
+                   std::map<MVertex *, double, MVertexPtrLessThan> &vSizes,
+                   std::set<MVertex *, MVertexPtrLessThan> &bndVertices)
 {
   for(int i = 0; i < 4; i++) {
     for(int j = i + 1; j < 4; j++) {
@@ -586,7 +586,7 @@ static void setLcs(MTetrahedron *t,
 
       // smallest tet edge
       if(bndVertices.find(vi) == bndVertices.end()) {
-        std::map<MVertex *, double, MVertexLessThanNum>::iterator iti =
+        std::map<MVertex *, double, MVertexPtrLessThan>::iterator iti =
           vSizes.find(vi);
 
         double const length =
@@ -596,7 +596,7 @@ static void setLcs(MTetrahedron *t,
       }
 
       if(bndVertices.find(vj) == bndVertices.end()) {
-        std::map<MVertex *, double, MVertexLessThanNum>::iterator itj =
+        std::map<MVertex *, double, MVertexPtrLessThan>::iterator itj =
           vSizes.find(vj);
 
         double const length =
@@ -613,9 +613,9 @@ static void completeTheSetOfFaces(GModel *model, std::set<GFace *> &faces_bound)
   std::set<GFace *> toAdd;
   for(GModel::fiter it = model->firstFace(); it != model->lastFace(); ++it) {
     if(faces_bound.find(*it) != faces_bound.end()) {
-      if((*it)->_compound.size()) {
-        for(std::size_t i = 0; i < (*it)->_compound.size(); ++i) {
-          GFace *gf = static_cast<GFace *>((*it)->_compound[i]);
+      if((*it)->compound.size()) {
+        for(std::size_t i = 0; i < (*it)->compound.size(); ++i) {
+          GFace *gf = static_cast<GFace *>((*it)->compound[i]);
           if(gf) toAdd.insert(gf);
         }
       }
@@ -697,19 +697,19 @@ void adaptMeshGRegion::operator()(GRegion *gr)
 
   typedef std::list<MTet4 *> CONTAINER;
   CONTAINER allTets;
-  for(unsigned int i = 0; i < gr->tetrahedra.size(); i++) {
+  for(std::size_t i = 0; i < gr->tetrahedra.size(); i++) {
     allTets.push_back(new MTet4(gr->tetrahedra[i], qm));
   }
   gr->tetrahedra.clear();
 
-  std::set<MFace, Less_Face> allEmbeddedFaces;
+  std::set<MFace, MFaceLessThan> allEmbeddedFaces;
   createAllEmbeddedFaces(gr, allEmbeddedFaces);
-  std::set<MEdge, Less_Edge> allEmbeddedEdges;
+  std::set<MEdge, MEdgeLessThan> allEmbeddedEdges;
   createAllEmbeddedEdges(gr, allEmbeddedEdges);
 
   connectTets(allTets.begin(), allTets.end(), &allEmbeddedFaces);
 
-  double t1 = Cpu();
+  double t1 = Cpu(), w1 = TimeOfDay();
   std::vector<MTet4 *> illegals;
   const int nbRanges = 10;
   int quality_ranges[nbRanges];
@@ -734,12 +734,12 @@ void adaptMeshGRegion::operator()(GRegion *gr)
         }
       }
     }
-    Msg::Info("Adaptation : START with %12.5E QBAD %12.5E QAVG %12.5E",
+    Msg::Info("Adaptation starts (volume = %g) with worst = %g / average = %g:",
               totalVolumeb, worst, avg / count);
     for(int i = 0; i < nbRanges; i++) {
       double low = (double)i / nbRanges;
       double high = (double)(i + 1) / nbRanges;
-      Msg::Info("Opti : %3.2f < QUAL < %3.2f : %9d elements ", low, high,
+      Msg::Info("%3.2f < quality < %3.2f: %9d elements ", low, high,
                 quality_ranges[i]);
     }
   }
@@ -764,8 +764,6 @@ void adaptMeshGRegion::operator()(GRegion *gr)
       }
     }
 
-    printf("nbCollapses = %d\n", nbCollapse);
-
     for(CONTAINER::iterator it = allTets.begin(); it != allTets.end(); ++it) {
       if(!(*it)->isDeleted()) {
         double qq = (*it)->getQuality();
@@ -810,7 +808,7 @@ void adaptMeshGRegion::operator()(GRegion *gr)
     if(!newTets.size()) break;
 
     // add all the new tets in the container
-    for(unsigned int i = 0; i < newTets.size(); i++) {
+    for(std::size_t i = 0; i < newTets.size(); i++) {
       if(!newTets[i]->isDeleted()) { allTets.push_back(newTets[i]); }
       else {
         delete newTets[i]->tet();
@@ -843,31 +841,30 @@ void adaptMeshGRegion::operator()(GRegion *gr)
         totalVolumeb += vol;
       }
     }
-    double t2 = Cpu();
-    Msg::Info("Opti : (%d,%d,%d) = %12.5E QBAD %12.5E QAVG %12.5E (%8.3f sec)",
-              nbESwap, nbFSwap, nbReloc, totalVolumeb, worst, avg / count,
-              t2 - t1);
+    double t2 = Cpu(), w2 = TimeOfDay();
+    Msg::Info("%d edge swaps, %d face swaps, %d node collapse, %d node "
+              "relocations (volume = %g): worst = %g / average = %g "
+              "(Wall %gs, CPU %gs)", nbESwap, nbFSwap, nbCollapse, nbReloc,
+              totalVolumeb, worst, avg / count, w2 - w1, t2 - t1);
     break;
   }
 
   int nbSlivers = 0;
-  for(unsigned int i = 0; i < illegals.size(); i++)
+  for(std::size_t i = 0; i < illegals.size(); i++)
     if(!(illegals[i]->isDeleted())) nbSlivers++;
 
   if(nbSlivers) {
-    Msg::Info(
-      "Opti : %d illegal tets are still in the mesh, trying to remove them",
-      nbSlivers);
+    Msg::Info("%d illegal tets are still in the mesh, trying to remove them",
+              nbSlivers);
   }
   else {
-    Msg::Info("Opti : no illegal tets in the mesh ;-)", nbSlivers);
+    Msg::Info("No illegal tets in the mesh :-)", nbSlivers);
   }
 
   for(int i = 0; i < nbRanges; i++) {
     double low = (double)i / nbRanges;
     double high = (double)(i + 1) / nbRanges;
-    Msg::Info("Opti : %3.2f < QUAL < %3.2f : %9d elements", low, high,
-              quality_ranges[i]);
+    Msg::Info("%3.2f < quality < %3.2f: %9d elements", low, high, quality_ranges[i]);
   }
 
   for(CONTAINER::iterator it = allTets.begin(); it != allTets.end(); ++it) {
@@ -888,25 +885,21 @@ void optimizeMesh(GRegion *gr, const qmTetrahedron::Measures &qm)
 
   if(qMin <= 0.0) return;
 
-  // well, this should not be true !!!
-  // if (gr->hexahedra.size() ||
-  //      gr->prisms.size() ||
-  //      gr->pyramids.size())return;
-  if(!gr->tetrahedra.size()) return;
+  if(gr->tetrahedra.empty()) return;
 
   typedef std::vector<MTet4 *> CONTAINER;
   CONTAINER allTets;
-  for(unsigned int i = 0; i < gr->tetrahedra.size(); i++) {
+  for(std::size_t i = 0; i < gr->tetrahedra.size(); i++) {
     MTet4 *t = new MTet4(gr->tetrahedra[i], qm);
     t->setOnWhat(gr);
     allTets.push_back(t);
   }
   gr->tetrahedra.clear();
 
-  std::set<MFace, Less_Face> allEmbeddedFaces;
+  std::set<MFace, MFaceLessThan> allEmbeddedFaces;
   createAllEmbeddedFaces(gr, allEmbeddedFaces);
 
-  std::set<MEdge, Less_Edge> allEmbeddedEdges;
+  std::set<MEdge, MEdgeLessThan> allEmbeddedEdges;
   createAllEmbeddedEdges(gr, allEmbeddedEdges);
 
   if(allEmbeddedFaces.empty()) {
@@ -918,7 +911,7 @@ void optimizeMesh(GRegion *gr, const qmTetrahedron::Measures &qm)
     connectTets(allTets.begin(), allTets.end(), &allEmbeddedFaces);
   }
 
-  double t1 = Cpu();
+  double t1 = Cpu(), w1 = TimeOfDay();
   std::vector<MTet4 *> illegals;
   const int nbRanges = 10;
   int quality_ranges[nbRanges];
@@ -943,39 +936,26 @@ void optimizeMesh(GRegion *gr, const qmTetrahedron::Measures &qm)
         }
       }
     }
-    Msg::Info("Opti : STARTS with %12.5E QBAD %12.5E QAVG %12.5E", totalVolumeb,
-              worst, avg / count);
+    Msg::Info("Optimization starts (volume = %g) with worst = %g / average = %g:",
+              totalVolumeb, worst, avg / count);
     for(int i = 0; i < nbRanges; i++) {
       double low = (double)i / nbRanges;
       double high = (double)(i + 1) / nbRanges;
-      Msg::Info("Opti : %3.2f < QUAL < %3.2f : %9d elements", low, high,
+      Msg::Info("%3.2f < quality < %3.2f : %9d elements", low, high,
                 quality_ranges[i]);
     }
   }
 
-  double sliverLimit = 0.04;
-
-  int nbESwap = 0, nbFSwap = 0, nbReloc = 0;
-
+  double sliverLimit = 0.001;
+  int nbESwap = 0, nbReloc = 0;
   double worstA = 0.0;
 
   while(1) {
     std::vector<MTet4 *> newTets;
-    /*    for (CONTAINER::iterator it = allTets.begin(); it != allTets.end();
-    ++it){ if (!(*it)->isDeleted()){ double qq = (*it)->getQuality(); if (qq <
-    qMin){ for (int i = 0; i < 4; i++){ if (faceSwap(newTets, *it, i, qm,
-    allEmbeddedFaces)){ nbFSwap++; break;
-            }
-          }
-        }
-      }
-    }
-    */
 
     illegals.clear();
     for(int i = 0; i < nbRanges; i++) quality_ranges[i] = 0;
 
-    //   printf("coucou\n");
     for(CONTAINER::iterator it = allTets.begin(); it != allTets.end(); ++it) {
       if(!(*it)->isDeleted()) {
         double qq = (*it)->getQuality();
@@ -1000,12 +980,11 @@ void optimizeMesh(GRegion *gr, const qmTetrahedron::Measures &qm)
         }
       }
     }
-    //    printf("coucou\n");
 
     if(!newTets.size()) { break; }
 
     // add all the new tets in the container
-    for(unsigned int i = 0; i < newTets.size(); i++) {
+    for(std::size_t i = 0; i < newTets.size(); i++) {
       if(!newTets[i]->isDeleted()) { allTets.push_back(newTets[i]); }
       else {
         delete newTets[i]->tet();
@@ -1014,14 +993,15 @@ void optimizeMesh(GRegion *gr, const qmTetrahedron::Measures &qm)
     }
 
     // relocate vertices
-    if(!gr->hexahedra.size() && !gr->prisms.size() && !gr->pyramids.size()) {
+    if(gr->hexahedra.empty() && gr->prisms.empty() && gr->pyramids.empty()) {
       for(CONTAINER::iterator it = allTets.begin(); it != allTets.end(); ++it) {
         if(!(*it)->isDeleted()) {
           double qq = (*it)->getQuality();
-          if(qq < qMin)
+          if(qq < qMin){
             for(int i = 0; i < 4; i++) {
               if(smoothVertex(*it, i, qm)) nbReloc++;
             }
+          }
         }
       }
     }
@@ -1041,26 +1021,27 @@ void optimizeMesh(GRegion *gr, const qmTetrahedron::Measures &qm)
       }
     }
 
-    double t2 = Cpu();
-    Msg::Info("Opti : (%d,%d,%d) = %12.5E QBAD %12.5E QAVG %12.5E (%8.3f sec)",
-              nbESwap, nbFSwap, nbReloc, totalVolumeb, worst, avg / count,
-              t2 - t1);
+    double t2 = Cpu(), w2 = TimeOfDay();
+    Msg::Info("%d edge swaps, %d node relocations (volume = %g): "
+              "worst = %g / average = %g (Wall %gs, CPU %gs)",
+              nbESwap, nbReloc, totalVolumeb, worst, avg / count,
+              w2 - w1, t2 - t1);
     if(worstA != 0.0 && worst - worstA < 1.e-6) break;
     worstA = worst;
   }
 
   if(illegals.size()) {
-    Msg::Info("Opti : %d ill-shaped tets are still in the mesh",
-              illegals.size());
+    Msg::Warning("%d ill-shaped tets are still in the mesh",
+                 illegals.size());
   }
   else {
-    Msg::Info("Opti : no ill-shaped tets in the mesh ;-)");
+    Msg::Info("No ill-shaped tets in the mesh :-)");
   }
 
   for(int i = 0; i < nbRanges; i++) {
     double low = (double)i / nbRanges;
     double high = (double)(i + 1) / nbRanges;
-    Msg::Info("Opti : %3.2f < QUAL < %3.2f : %9d elements", low, high,
+    Msg::Info("%3.2f < quality < %3.2f : %9d elements", low, high,
               quality_ranges[i]);
   }
 
@@ -1150,7 +1131,7 @@ double tetcircumcenter(double a[3], double b[3], double c[3], double d[3],
 static void memoryCleanup(MTet4Factory &myFactory,
                           std::set<MTet4 *, compareTet4Ptr> &allTets)
 {
-  //  int n1 = allTets.size();
+  // int n1 = allTets.size();
   std::set<MTet4 *, compareTet4Ptr>::iterator itd = allTets.begin();
   while(itd != allTets.end()) {
     if((*itd)->isDeleted()) {
@@ -1160,12 +1141,12 @@ static void memoryCleanup(MTet4Factory &myFactory,
     else
       itd++;
   }
-  //  Msg::Info("cleaning up the memory %d -> %d", n1, allTets.size());
+  // Msg::Info("Cleaning up memory %d -> %d", n1, allTets.size());
 }
 
-int isCavityCompatibleWithEmbeddedEdges(std::vector<MTet4 *> &cavity,
-                                        std::vector<faceXtet> &shell,
-                                        edgeContainerB &allEmbeddedEdges)
+static int isCavityCompatibleWithEmbeddedEdges(std::vector<MTet4 *> &cavity,
+                                               std::vector<faceXtet> &shell,
+                                               edgeContainerB &allEmbeddedEdges)
 {
   if (allEmbeddedEdges.empty())return 1;
   std::vector<MEdge> ed;
@@ -1191,35 +1172,9 @@ int isCavityCompatibleWithEmbeddedEdges(std::vector<MTet4 *> &cavity,
   return 1;
 }
 
-int isCavityCompatibleWithEmbeddedEdges(
-  std::list<MTet4 *> &cavity, std::vector<faceXtet> &shell,
-  std::set<MEdge, Less_Edge> &allEmbeddedEdges)
-{
-  if (allEmbeddedEdges.empty())return 1;
-  std::set<MEdge, Less_Edge> ed;
-  for(std::vector<faceXtet>::iterator it = shell.begin(); it != shell.end();
-      it++) {
-    ed.insert(MEdge(it->v[0], it->v[1]));
-    ed.insert(MEdge(it->v[1], it->v[2]));
-    ed.insert(MEdge(it->v[2], it->v[0]));
-  }
-
-  for(std::list<MTet4 *>::iterator itc = cavity.begin(); itc != cavity.end();
-      ++itc) {
-    for(int j = 0; j < 6; j++) {
-      MEdge e = (*itc)->tet()->getEdge(j);
-      if(ed.find(e) == ed.end() &&
-         allEmbeddedEdges.find(e) != allEmbeddedEdges.end()) {
-        return 0;
-      }
-    }
-  }
-  return 1;
-}
-
-int isCavityCompatibleWithEmbeddedFace(
+static int isCavityCompatibleWithEmbeddedFace(
   const std::vector<MTet4 *> &cavity, const std::vector<faceXtet> &shell,
-  const std::set<MFace, Less_Face> &allEmbeddedFaces)
+  const std::set<MFace, MFaceLessThan> &allEmbeddedFaces)
 {
   if (allEmbeddedFaces.empty())return 1;
   std::vector<MFace> shellFaces;
@@ -1248,7 +1203,7 @@ int isCavityCompatibleWithEmbeddedFace(
 
 static void _deleteUnusedVertices(GRegion *gr)
 {
-  std::set<MVertex *> allverts;
+  std::set<MVertex *, MVertexPtrLessThan> allverts;
   for(std::size_t i = 0; i < gr->tetrahedra.size(); i++) {
     for(int j = 0; j < 4; j++) {
       if(gr->tetrahedra[i]->getVertex(j)->onWhat() == gr)
@@ -1256,17 +1211,17 @@ static void _deleteUnusedVertices(GRegion *gr)
     }
   }
   for(std::size_t i = 0; i < gr->mesh_vertices.size(); i++) {
-    // FIXME: investiagte crashes on exit (e.g. t16.geo)
-    //if(allverts.find(gr->mesh_vertices[i]) == allverts.end())
-    //  delete gr->mesh_vertices[i];
+    // FIXME: investigate crash on exit (e.g. t16.geo)
+    // if(allverts.find(gr->mesh_vertices[i]) == allverts.end())
+    //   delete gr->mesh_vertices[i];
   }
   gr->mesh_vertices.clear();
   gr->mesh_vertices.insert(gr->mesh_vertices.end(), allverts.begin(),
                            allverts.end());
 }
 
-void insertVerticesInRegion(GRegion *gr, int maxVert, bool _classify,
-                            splitQuadRecovery *sqr)
+void insertVerticesInRegion(GRegion *gr, int maxIter, double worstTetRadiusTarget,
+                            bool _classify, splitQuadRecovery *sqr)
 {
 
 #ifdef DEBUG_BOUNDARY_RECOVERY
@@ -1280,15 +1235,15 @@ void insertVerticesInRegion(GRegion *gr, int maxVert, bool _classify,
 
   // leave this in a block so the map gets deallocated directly
   {
-    std::map<MVertex *, double, MVertexLessThanNum> vSizesMap;
-    std::set<MVertex *, MVertexLessThanNum> bndVertices;
+    std::map<MVertex *, double, MVertexPtrLessThan> vSizesMap;
+    std::set<MVertex *, MVertexPtrLessThan> bndVertices;
 
     for(GModel::riter rit = gr->model()->firstRegion();
         rit != gr->model()->lastRegion(); ++rit) {
       std::vector<GEdge *> const &e = (*rit)->embeddedEdges();
       for(std::vector<GEdge *>::const_iterator it = e.begin(); it != e.end();
           ++it) {
-        for(unsigned int i = 0; i < (*it)->lines.size(); i++) {
+        for(std::size_t i = 0; i < (*it)->lines.size(); i++) {
           MVertex *vi = (*it)->lines[i]->getVertex(0);
           MVertex *vj = (*it)->lines[i]->getVertex(1);
           double dx = vi->x() - vj->x();
@@ -1296,9 +1251,9 @@ void insertVerticesInRegion(GRegion *gr, int maxVert, bool _classify,
           double dz = vi->z() - vj->z();
           double l = std::sqrt(dx * dx + dy * dy + dz * dz);
 
-          std::map<MVertex *, double, MVertexLessThanNum>::iterator iti =
+          std::map<MVertex *, double, MVertexPtrLessThan>::iterator iti =
             vSizesMap.find(vi);
-          std::map<MVertex *, double, MVertexLessThanNum>::iterator itj =
+          std::map<MVertex *, double, MVertexPtrLessThan>::iterator itj =
             vSizesMap.find(vj);
 
           // smallest tet edge
@@ -1315,7 +1270,7 @@ void insertVerticesInRegion(GRegion *gr, int maxVert, bool _classify,
           it != vertices.end(); ++it) {
         MVertex *v = (*it)->getMeshVertex(0);
         double l = (*it)->prescribedMeshSizeAtVertex();
-        std::map<MVertex *, double, MVertexLessThanNum>::iterator itv =
+        std::map<MVertex *, double, MVertexPtrLessThan>::iterator itv =
           vSizesMap.find(v);
         if(itv == vSizesMap.end() || itv->second > l) vSizesMap[v] = l;
       }
@@ -1324,14 +1279,14 @@ void insertVerticesInRegion(GRegion *gr, int maxVert, bool _classify,
     for(GModel::fiter it = gr->model()->firstFace();
         it != gr->model()->lastFace(); ++it) {
       GFace *gf = *it;
-      for(unsigned int i = 0; i < gf->triangles.size(); i++) {
+      for(std::size_t i = 0; i < gf->triangles.size(); i++) {
         setLcs(gf->triangles[i], vSizesMap, bndVertices);
       }
     }
-    for(unsigned int i = 0; i < gr->tetrahedra.size(); i++)
+    for(std::size_t i = 0; i < gr->tetrahedra.size(); i++)
       setLcs(gr->tetrahedra[i], vSizesMap, bndVertices);
 
-    for(std::map<MVertex *, double, MVertexLessThanNum>::iterator it =
+    for(std::map<MVertex *, double, MVertexPtrLessThan>::iterator it =
           vSizesMap.begin();
         it != vSizesMap.end(); ++it) {
       it->first->setIndex(NUM++);
@@ -1340,7 +1295,7 @@ void insertVerticesInRegion(GRegion *gr, int maxVert, bool _classify,
     }
   }
 
-  for(unsigned int i = 0; i < gr->tetrahedra.size(); i++) {
+  for(std::size_t i = 0; i < gr->tetrahedra.size(); i++) {
     gr->tetrahedra[i]->setVolumePositive();
     allTets.insert(myFactory.Create(gr->tetrahedra[i], vSizes, vSizesBGM));
   }
@@ -1363,19 +1318,18 @@ void insertVerticesInRegion(GRegion *gr, int maxVert, bool _classify,
         std::list<MTet4 *> theRegion;
         std::set<GFace *> faces_bound;
         GRegion *bidon = (GRegion *)123;
-        double _t1 = Cpu();
+        double _t1 = Cpu(), _w1 = TimeOfDay();
         Msg::Debug("start with a non classified tet");
         non_recursive_classify(*it, theRegion, faces_bound, bidon, gr->model(),
                                search);
-        double _t2 = Cpu();
-        Msg::Debug(
-          "found %d tets with %d faces (%g sec for the classification)",
-          theRegion.size(), faces_bound.size(), _t2 - _t1);
+        double _t2 = Cpu(), _w2 = TimeOfDay();
+        Msg::Debug("Found %d tets with %d faces (Wall %gs, CPU %gs)",
+                   theRegion.size(), faces_bound.size(), _w2 - _w1, _t2 - _t1);
         GRegion *myGRegion =
           getRegionFromBoundingFaces(gr->model(), faces_bound);
         if(myGRegion) { // a geometrical region associated to the list of faces
                         // has been found
-          Msg::Info("Found region %d", myGRegion->tag());
+          Msg::Info("Found volume %d", myGRegion->tag());
           for(std::list<MTet4 *>::iterator it2 = theRegion.begin();
               it2 != theRegion.end(); ++it2) {
             (*it2)->setOnWhat(myGRegion);
@@ -1417,7 +1371,7 @@ void insertVerticesInRegion(GRegion *gr, int maxVert, bool _classify,
     (*it)->setNeigh(3, 0);
   }
   // store all embedded faces
-  std::set<MFace, Less_Face> allEmbeddedFaces;
+  std::set<MFace, MFaceLessThan> allEmbeddedFaces;
   edgeContainerB allEmbeddedEdges;
   for(GModel::riter it = gr->model()->firstRegion();
       it != gr->model()->lastRegion(); ++it) {
@@ -1434,13 +1388,12 @@ void insertVerticesInRegion(GRegion *gr, int maxVert, bool _classify,
   int COUNT_MISS_1 = 0;
   int COUNT_MISS_2 = 0;
 
-  double t1 = Cpu();
+  double t1 = TimeOfDay();
 
   // main loop in Delaunay inserstion starts here
 
   while(1) {
-    if(COUNT_MISS_2 > 100000) break;
-    if(ITER >= maxVert) break;
+    if(maxIter > 0 && ITER >= maxIter) break;
     if(allTets.empty()) {
       Msg::Error("No tetrahedra in region %d", gr->tag());
       break;
@@ -1454,10 +1407,9 @@ void insertVerticesInRegion(GRegion *gr, int maxVert, bool _classify,
     }
     else {
       if(ITER++ % 500 == 0)
-        Msg::Info(
-          "%d points created - Worst tet radius is %g (PTS removed %d %d)",
-          REALCOUNT, worst->getRadius(), COUNT_MISS_1, COUNT_MISS_2);
-      if(worst->getRadius() < 1) break;
+        Msg::Info("It. %d - %d nodes created - worst tet radius %g (nodes removed %d %d)",
+                  ITER - 1, REALCOUNT, worst->getRadius(), COUNT_MISS_1, COUNT_MISS_2);
+      if(worst->getRadius() < worstTetRadiusTarget) break;
       double center[3];
       double uvw[3];
       MTetrahedron *base = worst->tet();
@@ -1494,8 +1446,7 @@ void insertVerticesInRegion(GRegion *gr, int maxVert, bool _classify,
 
       if(FOUND && (!allEmbeddedEdges.empty() || !allEmbeddedFaces.empty())) {
         FOUND =
-          isCavityCompatibleWithEmbeddedEdges(cavity, shell,
-                                              allEmbeddedEdges) &&
+          isCavityCompatibleWithEmbeddedEdges(cavity, shell, allEmbeddedEdges) &&
           isCavityCompatibleWithEmbeddedFace(cavity, shell, allEmbeddedFaces);
       }
 
@@ -1574,15 +1525,15 @@ void insertVerticesInRegion(GRegion *gr, int maxVert, bool _classify,
   }
 
   memoryCleanup(myFactory, allTets);
-  double t2 = Cpu();
+  double t2 = TimeOfDay();
   double dt = (t2 - t1);
   int COUNT_MISS = COUNT_MISS_1 + COUNT_MISS_2;
-  Msg::Info("3D point insertion terminated (%d points created):",
+  Msg::Info("3D refinement terminated (%d nodes total):",
             (int)vSizes.size());
   Msg::Info(" - %d Delaunay cavities modified for star shapeness",
             NB_CORRECTION_OF_CAVITY);
-  Msg::Info(" - %d points could not be inserted", COUNT_MISS);
-  Msg::Info(" - %d tetrahedra created in %g sec. (%d tets/sec.)",
+  Msg::Info(" - %d nodes could not be inserted", COUNT_MISS);
+  Msg::Info(" - %d tetrahedra created in %g sec. (%d tets/s)",
             allTets.size(), dt, (int)(allTets.size() / dt));
 
   // relocate vertices
@@ -1619,8 +1570,10 @@ void insertVerticesInRegion(GRegion *gr, int maxVert, bool _classify,
 void delaunayMeshIn3D(std::vector<MVertex *> &v,
                       std::vector<MTetrahedron *> &result)
 {
-  double t1 = Cpu();
+  Msg::Info("Tetrahedrizing %d nodes...", v.size());
+  double t1 = Cpu(), w1 = TimeOfDay();
   delaunayTriangulation(1, 1, v, result);
-  double t2 = Cpu();
-  Msg::Info("Tetrahedrization of %d points in %g seconds", v.size(), t2 - t1);
+  double t2 = Cpu(), w2 = TimeOfDay();
+  Msg::Info("Done tetrahedrizing %d nodes (Wall %gs, CPU %gs)",
+            v.size(), w2 - w1, t2 - t1);
 }
