@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2019 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2020 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
@@ -51,6 +51,8 @@ static void addHexahedron(MVertex *v1, MVertex *v2, MVertex *v3, MVertex *v4,
 static void createPriPyrTet(std::vector<MVertex *> &v, GRegion *to,
                             MElement *source)
 {
+  static int warningReg = 0;
+
   int dup[3];
   int j = 0;
   for(int i = 0; i < 3; i++)
@@ -74,13 +76,19 @@ static void createPriPyrTet(std::vector<MVertex *> &v, GRegion *to,
   }
   else {
     addPrism(v[0], v[1], v[2], v[3], v[4], v[5], to);
-    if(j) Msg::Error("Degenerated prism in extrusion of volume %d", to->tag());
+    if(j && warningReg != to->tag()) {
+      warningReg = to->tag();
+      Msg::Warning("Degenerated prism in extrusion of volume %d", to->tag());
+    }
   }
 }
 
 static void createHexPri(std::vector<MVertex *> &v, GRegion *to,
                          MElement *source)
 {
+  static int errorReg = 0;
+  static int warningReg = 0;
+
   int dup[4];
   int j = 0;
   for(int i = 0; i < 4; i++)
@@ -95,13 +103,17 @@ static void createHexPri(std::vector<MVertex *> &v, GRegion *to,
       addPrism(v[0], v[3], v[4], v[1], v[2], v[5], to);
     else if(dup[0] == 0 && dup[1] == 3)
       addPrism(v[0], v[1], v[5], v[3], v[2], v[6], to);
-    else
-      Msg::Error("Uncoherent hexahedron in extrusion of volume %d", to->tag());
+    else if(to->tag() != errorReg) {
+      errorReg = to->tag();
+      Msg::Error("Wrong hexahedron in extrusion of volume %d", to->tag());
+    }
   }
   else {
     addHexahedron(v[0], v[1], v[2], v[3], v[4], v[5], v[6], v[7], to);
-    if(j)
-      Msg::Error("Degenerated hexahedron in extrusion of volume %d", to->tag());
+    if(j && warningReg != to->tag()) {
+      warningReg = to->tag();
+      Msg::Warning("Degenerated hexahedron in extrusion of volume %d", to->tag());
+    }
   }
 }
 
@@ -174,9 +186,11 @@ static void extrudeMesh(GFace *from, GRegion *to, MVertexRTree &pos)
         double x = v->x(), y = v->y(), z = v->z();
         ep->Extrude(j, k + 1, x, y, z);
         if(j != ep->mesh.NbLayer - 1 || k != ep->mesh.NbElmLayer[j] - 1) {
-          MVertex *newv = new MVertex(x, y, z, to);
-          to->mesh_vertices.push_back(newv);
-          pos.insert(newv);
+          if(!pos.find(x, y, z)) {
+            MVertex *newv = new MVertex(x, y, z, to);
+            to->mesh_vertices.push_back(newv);
+            pos.insert(newv);
+          }
         }
       }
     }
@@ -223,23 +237,23 @@ static void extrudeMesh(GFace *from, GRegion *to, MVertexRTree &pos)
 static void insertAllVertices(GRegion *gr, MVertexRTree &pos)
 {
   pos.insert(gr->mesh_vertices);
+  std::vector<MVertex*> embedded = gr->getEmbeddedMeshVertices();
+  pos.insert(embedded);
   std::vector<GFace *> faces = gr->faces();
-  std::vector<GFace *>::iterator itf = faces.begin();
-  while(itf != faces.end()) {
+  for(std::vector<GFace *>::iterator itf = faces.begin(); itf != faces.end();
+      itf++) {
     pos.insert((*itf)->mesh_vertices);
     std::vector<MVertex *> embedded = (*itf)->getEmbeddedMeshVertices();
     pos.insert(embedded);
     std::vector<GEdge *> const &edges = (*itf)->edges();
-    std::vector<GEdge *>::const_iterator ite = edges.begin();
-    while(ite != edges.end()) {
+    for(std::vector<GEdge *>::const_iterator ite = edges.begin(); ite != edges.end();
+        ite++) {
       pos.insert((*ite)->mesh_vertices);
       if((*ite)->getBeginVertex())
         pos.insert((*ite)->getBeginVertex()->mesh_vertices);
       if((*ite)->getEndVertex())
-         pos.insert((*ite)->getEndVertex()->mesh_vertices);
-      ++ite;
+        pos.insert((*ite)->getEndVertex()->mesh_vertices);
     }
-    ++itf;
   }
 }
 
@@ -253,7 +267,7 @@ void meshGRegionExtruded::operator()(GRegion *gr)
 
   if(!ep || !ep->mesh.ExtrudeMesh || ep->geo.Mode != EXTRUDED_ENTITY) return;
 
-  Msg::Info("Meshing volume %d (extruded)", gr->tag());
+  Msg::Info("Meshing volume %d (Extruded)", gr->tag());
 
   // destroy the mesh if it exists
   deMeshGRegion dem;
@@ -304,7 +318,8 @@ static void deleteEdge(MVertex *v1, MVertex *v2,
 
 // subdivide the 3 lateral faces of each prism
 static void phase1(GRegion *gr, MVertexRTree &pos,
-                   std::set<std::pair<MVertex *, MVertex *> > &edges)
+                   std::set<std::pair<MVertex *, MVertex *> > &edges,
+                   int ntry)
 {
   ExtrudeParams *ep = gr->meshAttributes.extrude;
   GFace *from = gr->model()->getFaceByTag(std::abs(ep->geo.Source));
@@ -315,27 +330,28 @@ static void phase1(GRegion *gr, MVertexRTree &pos,
       for(int k = 0; k < ep->mesh.NbElmLayer[j]; k++) {
         std::vector<MVertex *> v;
         if(getExtrudedVertices(from->triangles[i], ep, j, k, pos, v) == 6) {
-#if 0 // old
-          if(!edgeExists(v[0], v[4], edges))
-            createEdge(v[1], v[3], edges);
-          if(!edgeExists(v[4], v[2], edges))
-            createEdge(v[1], v[5], edges);
-          if(!edgeExists(v[3], v[2], edges))
-            createEdge(v[0], v[5], edges);
-#else // new from Michel Benhamou
-          if(v[1] < v[0])
-            createEdge(v[1], v[3], edges);
-          else
-            createEdge(v[0], v[4], edges);
-          if(v[1] < v[2])
-            createEdge(v[1], v[5], edges);
-          else
-            createEdge(v[4], v[2], edges);
-          if(v[0] < v[2])
-            createEdge(v[0], v[5], edges);
-          else
-            createEdge(v[3], v[2], edges);
-#endif
+          if(ntry == 1){
+            if(!edgeExists(v[0], v[4], edges))
+              createEdge(v[1], v[3], edges);
+            if(!edgeExists(v[4], v[2], edges))
+              createEdge(v[1], v[5], edges);
+            if(!edgeExists(v[3], v[2], edges))
+              createEdge(v[0], v[5], edges);
+          }
+          else{ // from Michel Benhamou
+            if(v[1]->getNum() < v[0]->getNum())
+              createEdge(v[1], v[3], edges);
+            else
+              createEdge(v[0], v[4], edges);
+            if(v[1]->getNum() < v[2]->getNum())
+              createEdge(v[1], v[5], edges);
+            else
+              createEdge(v[4], v[2], edges);
+            if(v[0]->getNum() < v[2]->getNum())
+              createEdge(v[0], v[5], edges);
+            else
+              createEdge(v[3], v[2], edges);
+          }
         }
       }
     }
@@ -497,54 +513,83 @@ int SubdivideExtrudedMesh(GModel *m)
   }
 
   if(regions.empty()) return 0;
+
   Msg::Info("Subdividing extruded mesh");
 
-  // create edges on lateral sides of "prisms"
   std::set<std::pair<MVertex *, MVertex *> > edges;
-  for(std::size_t i = 0; i < regions.size(); i++)
-    phase1(regions[i], pos, edges);
 
-  // swap lateral edges to make them "tet-compatible"
-  int j = 0, swap;
-  std::set<std::pair<MVertex *, MVertex *> > edges_swap;
-  do {
-    swap = 0;
+  for(int ntry = 1; ntry <= 2; ntry++) {
+    // create edges on lateral sides of "prisms"
     for(std::size_t i = 0; i < regions.size(); i++)
-      phase2(regions[i], pos, edges, edges_swap, swap);
-    Msg::Info("Swapping %d", swap);
-    if(j && j == swap) {
-      Msg::Error("Unable to subdivide extruded mesh: change surface mesh or");
-      Msg::Error("recombine extrusion instead");
-      return -1;
-    }
-    j = swap;
-  } while(swap);
+      phase1(regions[i], pos, edges, ntry);
+    // swap lateral edges to make them "tet-compatible"
+    int j = 0, swap;
+    std::set<std::pair<MVertex *, MVertex *> > edges_swap;
+    do {
+      swap = 0;
+      for(std::size_t i = 0; i < regions.size(); i++)
+        phase2(regions[i], pos, edges, edges_swap, swap);
+      Msg::Info("Swapping %d", swap);
+      if(j && j == swap) {
+        if(ntry == 1) {
+          Msg::Info("Subdivision failed, trying alternative split");
+          edges.clear();
+          break;
+        }
+        else{
+          Msg::Error("Unable to subdivide extruded mesh: change surface mesh or");
+          Msg::Error("recombine extrusion instead");
+          return -1;
+        }
+      }
+      j = swap;
+    } while(swap);
+    if(!swap) break;
+  }
 
   // delete volume elements and create tetrahedra instead
   for(std::size_t i = 0; i < regions.size(); i++) {
     GRegion *gr = regions[i];
-
     for(std::size_t i = 0; i < gr->tetrahedra.size(); i++)
       delete gr->tetrahedra[i];
     gr->tetrahedra.clear();
     for(std::size_t i = 0; i < gr->hexahedra.size(); i++)
       delete gr->hexahedra[i];
     gr->hexahedra.clear();
-    for(std::size_t i = 0; i < gr->prisms.size(); i++) delete gr->prisms[i];
+    for(std::size_t i = 0; i < gr->prisms.size(); i++)
+      delete gr->prisms[i];
     gr->prisms.clear();
     for(std::size_t i = 0; i < gr->pyramids.size(); i++)
       delete gr->pyramids[i];
     gr->pyramids.clear();
     phase3(gr, pos, edges);
+  }
 
-    // re-Extrude bounding surfaces using edges as constraint
-    std::vector<GFace *> faces = gr->faces();
-    for(std::vector<GFace *>::iterator it = faces.begin(); it != faces.end();
-        it++) {
-      ExtrudeParams *ep = (*it)->meshAttributes.extrude;
+  // remesh bounding surfaces, to make them compatible with the volume mesh
+  std::set<GFace*> faces;
+  for(std::size_t i = 0; i < regions.size(); i++) {
+    GRegion *gr = regions[i];
+    std::vector<GFace *> f = gr->faces();
+    for(std::size_t i = 0; i < f.size(); i++){
+      ExtrudeParams *ep = f[i]->meshAttributes.extrude;
+      // TODO: this does not yet handle swapping of edges in COPIED_ENTITY
+      // surfaces, whose mesh is copied during extrusion (the "top"
+      // surface). PS: we will need to distinguish the old boundary layer case
+      // if we include these "copied" meshes
       if(ep && ep->mesh.ExtrudeMesh && ep->geo.Mode == EXTRUDED_ENTITY &&
          !ep->mesh.Recombine) {
-        GFace *gf = *it;
+        f[i]->meshStatistics.status = GFace::PENDING;
+        faces.insert(f[i]);
+      }
+    }
+  }
+  int nIter = 0;
+  while(1) {
+    int nPending = 0;
+    for(std::set<GFace *>::iterator it = faces.begin(); it != faces.end();
+        it++) {
+      GFace *gf = *it;
+      if(gf->meshStatistics.status == GFace::PENDING){
         Msg::Info("Remeshing surface %d", gf->tag());
         for(std::size_t i = 0; i < gf->triangles.size(); i++)
           delete gf->triangles[i];
@@ -553,8 +598,14 @@ int SubdivideExtrudedMesh(GModel *m)
           delete gf->quadrangles[i];
         gf->quadrangles.clear();
         MeshExtrudedSurface(gf, &edges);
+        nPending++;
       }
     }
+    if(!nPending) break;
+    if(nIter++ > 10){
+      Msg::Warning("Could not remesh all subdivided surfaces");
+      break;
+    }
   }
 
 #if defined(HAVE_QUADTRI)
