@@ -1,10 +1,10 @@
-// Gmsh - Copyright (C) 1997-2019 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2020 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
 
-#ifndef _CONJUGATEGRADIENTS
-#define _CONJUGATEGRADIENTS
+#ifndef CONJUGATE_GRADIENTS_H
+#define CONJUGATE_GRADIENTS_H
 
 #include <vector>
 
