@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2019 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2020 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
@@ -13,21 +13,21 @@
 namespace {
 
   template <class T>
-  void calcMapFromIdealElement(int type, T &gSMatX, T &gSMatY, T &gSMatZ)
+  void calcMapFromIdealElement(int type, T &dSVec_dX, T &dSVec_dY, T &dSVec_dZ)
   {
     // 2D scaling
     switch(type) {
     case TYPE_QUA:
     case TYPE_HEX:
     case TYPE_PYR: { // Quad, hex, pyramid -> square with side of length 1
-      gSMatX.scale(2.);
-      gSMatY.scale(2.);
+      dSVec_dX.scale(2.);
+      dSVec_dY.scale(2.);
       break;
     }
     default: { // Tri, tet, prism: equilateral tri with side of length 1
       static const double cTri[2] = {-1. / std::sqrt(3.), 2. / std::sqrt(3.)};
-      gSMatY.scale(cTri[1]);
-      gSMatY.axpy(gSMatX, cTri[0]);
+      dSVec_dY.scale(cTri[1]);
+      dSVec_dY.axpy(dSVec_dX, cTri[0]);
       break;
     }
     }
@@ -36,21 +36,21 @@ namespace {
     switch(type) {
     case TYPE_HEX:
     case TYPE_PRI: { // Hex, prism -> side of length 1 in z
-      gSMatZ.scale(2.);
+      dSVec_dZ.scale(2.);
       break;
     }
     case TYPE_PYR: { // Pyramid -> height sqrt(2.)/2
       static const double cPyr = sqrt(2.);
-      gSMatZ.scale(cPyr);
+      dSVec_dZ.scale(cPyr);
       break;
     }
     case TYPE_TET: { // Tet: take into account (x, y) scaling to obtain regular
                      // tet
       static const double cTet[3] = {-3. / 2 / std::sqrt(6.),
                                      -1. / 2 / std::sqrt(2.), std::sqrt(1.5)};
-      gSMatZ.scale(cTet[2]);
-      gSMatZ.axpy(gSMatX, cTet[0]);
-      gSMatZ.axpy(gSMatY, cTet[1]);
+      dSVec_dZ.scale(cTet[2]);
+      dSVec_dZ.axpy(dSVec_dX, cTet[0]);
+      dSVec_dZ.axpy(dSVec_dY, cTet[1]);
       break;
     }
     }
@@ -65,71 +65,72 @@ namespace {
            M13 * (M21 * M32 - M22 * M31);
   }
 
-  // Compute signed Jacobian and its gradients w.r.t.  node positions, at one
-  // location in a 1D element
+  // Compute (signed) Jacobian determinant, and its partial derivative w.r.t.
+  // nodes coordinates, at sampling point 'i' of a 1D element (for which columns
+  // 2 and 3 of the Jacobian matrix are constant).
   inline void calcJDJ1D(double dxdX, double dxdY, double dxdZ, double dydX,
                         double dydY, double dydZ, double dzdX, double dzdY,
                         double dzdZ, int i, int numMapNodes,
-                        const fullMatrix<double> &gSMatX,
+                        const fullMatrix<double> &dShapeMat_dX,
                         fullMatrix<double> &JDJ)
   {
     for(int j = 0; j < numMapNodes; j++) {
-      const double &dPhidX = gSMatX(i, j);
-      JDJ(i, j) = dPhidX * dydY * dzdZ + dPhidX * dzdY * dydZ;
-      JDJ(i, j + numMapNodes) = dPhidX * dzdY * dxdZ - dPhidX * dxdY * dzdZ;
-      JDJ(i, j + 2 * numMapNodes) = dPhidX * dxdY * dydZ - dPhidX * dydY * dxdZ;
+      const double &dPhidX = dShapeMat_dX(i, j);
+      JDJ(i, j) =                   dPhidX * (dydY * dzdZ - dzdY * dydZ);
+      JDJ(i, j + numMapNodes) =     dPhidX * (dzdY * dxdZ - dxdY * dzdZ);
+      JDJ(i, j + 2 * numMapNodes) = dPhidX * (dxdY * dydZ - dydY * dxdZ);
     }
     JDJ(i, 3 * numMapNodes) =
       calcDet3D(dxdX, dxdY, dxdZ, dydX, dydY, dydZ, dzdX, dzdY, dzdZ);
   }
 
-  // Compute signed Jacobian and its gradients w.r.t.  node positions, at one
-  // location in a 2D element
+  // Compute (signed) Jacobian determinant, and its partial derivative w.r.t.
+  // nodes coordinates, at sampling point 'i' of a 2D element (for which column
+  // 3 of the Jacobian matrix is constant).
   inline void calcJDJ2D(double dxdX, double dxdY, double dxdZ, double dydX,
                         double dydY, double dydZ, double dzdX, double dzdY,
                         double dzdZ, int i, int numMapNodes,
-                        const fullMatrix<double> &gSMatX,
-                        const fullMatrix<double> &gSMatY,
+                        const fullMatrix<double> &dShapeMat_dX,
+                        const fullMatrix<double> &dShapeMat_dY,
                         fullMatrix<double> &JDJ)
   {
     for(int j = 0; j < numMapNodes; j++) {
-      const double &dPhidX = gSMatX(i, j);
-      const double &dPhidY = gSMatY(i, j);
-      JDJ(i, j) = dPhidX * dydY * dzdZ + dzdX * dPhidY * dydZ +
-                  dPhidX * dzdY * dydZ - dydX * dPhidY * dzdZ;
-      JDJ(i, j + numMapNodes) = dxdX * dPhidY * dzdZ + dPhidX * dzdY * dxdZ -
-                                dzdX * dPhidY * dxdZ - dPhidX * dxdY * dzdZ;
-      JDJ(i, j + 2 * numMapNodes) = dPhidX * dxdY * dydZ +
-                                    dydX * dPhidY * dxdZ -
-                                    dPhidX * dydY * dxdZ - dxdX * dPhidY * dydZ;
+      const double &dPhidX = dShapeMat_dX(i, j);
+      const double &dPhidY = dShapeMat_dY(i, j);
+      JDJ(i, j) =   dPhidX * (dydY * dzdZ - dzdY * dydZ)
+                  + dPhidY * (dzdX * dydZ - dydX * dzdZ);
+      JDJ(i, j + numMapNodes) =   dPhidX * (dzdY * dxdZ - dxdY * dzdZ)
+                                + dPhidY * (dxdX * dzdZ - dzdX * dxdZ);
+      JDJ(i, j + 2 * numMapNodes) =   dPhidX * (dxdY * dydZ - dydY * dxdZ)
+                                    + dPhidY * (dydX * dxdZ - dxdX * dydZ);
     }
     JDJ(i, 3 * numMapNodes) =
       calcDet3D(dxdX, dxdY, dxdZ, dydX, dydY, dydZ, dzdX, dzdY, dzdZ);
   }
 
-  // Compute signed Jacobian and its gradients w.r.t.  node positions, at one
-  // location in a 3D element
+  // Compute (signed) Jacobian determinant, and its partial derivative w.r.t.
+  // nodes coordinates, at sampling point 'i' of a 3D element
   inline void calcJDJ3D(double dxdX, double dxdY, double dxdZ, double dydX,
                         double dydY, double dydZ, double dzdX, double dzdY,
                         double dzdZ, int i, int numMapNodes,
-                        const fullMatrix<double> &gSMatX,
-                        const fullMatrix<double> &gSMatY,
-                        const fullMatrix<double> &gSMatZ,
+                        const fullMatrix<double> &dShapeMat_dX,
+                        const fullMatrix<double> &dShapeMat_dY,
+                        const fullMatrix<double> &dShapeMat_dZ,
                         fullMatrix<double> &JDJ)
   {
     for(int j = 0; j < numMapNodes; j++) {
-      const double &dPhidX = gSMatX(i, j);
-      const double &dPhidY = gSMatY(i, j);
-      const double &dPhidZ = gSMatZ(i, j);
-      JDJ(i, j) = dPhidX * dydY * dzdZ + dzdX * dPhidY * dydZ +
-                  dydX * dzdY * dPhidZ - dzdX * dydY * dPhidZ -
-                  dPhidX * dzdY * dydZ - dydX * dPhidY * dzdZ;
-      JDJ(i, j + numMapNodes) = dxdX * dPhidY * dzdZ + dzdX * dxdY * dPhidZ +
-                                dPhidX * dzdY * dxdZ - dzdX * dPhidY * dxdZ -
-                                dxdX * dzdY * dPhidZ - dPhidX * dxdY * dzdZ;
-      JDJ(i, j + 2 * numMapNodes) =
-        dxdX * dydY * dPhidZ + dPhidX * dxdY * dydZ + dydX * dPhidY * dxdZ -
-        dPhidX * dydY * dxdZ - dxdX * dPhidY * dydZ - dydX * dxdY * dPhidZ;
+      const double &dPhidX = dShapeMat_dX(i, j);
+      const double &dPhidY = dShapeMat_dY(i, j);
+      const double &dPhidZ = dShapeMat_dZ(i, j);
+      JDJ(i, j) =   dPhidX * (dydY * dzdZ - dzdY * dydZ)
+                  + dPhidY * (dzdX * dydZ - dydX * dzdZ)
+                  + dPhidZ * (dydX * dzdY - dzdX * dydY);
+      JDJ(i, j + numMapNodes) =   dPhidX * (dzdY * dxdZ - dxdY * dzdZ)
+                                + dPhidY * (dxdX * dzdZ - dzdX * dxdZ)
+                                + dPhidZ * (dzdX * dxdY - dxdX * dzdY);
+      JDJ(i, j + 2 * numMapNodes) =   dPhidX * (dxdY * dydZ - dydY * dxdZ)
+                                    + dPhidY * (dydX * dxdZ - dxdX * dydZ)
+                                    + dPhidZ * (dxdX * dydY - dydX * dxdY);
     }
     JDJ(i, 3 * numMapNodes) =
       calcDet3D(dxdX, dxdY, dxdZ, dydX, dydY, dydZ, dzdX, dzdY, dzdZ);
@@ -137,104 +138,108 @@ namespace {
 
 } // namespace
 
-GradientBasis::GradientBasis(int elementTag, FuncSpaceData data)
-  : _elementTag(elementTag), _data(data)
+// GradientBasis contains the information for computing the partial
+// derivatives of the mapping components w.r.t. to each reference coordinate
+// for a given element type.
+// Those partial derivatives are the components of the Jacobian matrix,
+// and the gradients are the rows of the Jacobian matrix.
+GradientBasis::GradientBasis(int elementTag, FuncSpaceData fsdata)
+  : _elementTag(elementTag), _data(fsdata)
 {
-  // Matrix gradShapeMatX, when multiplied by Lagrange coefficients,
-  // gives the first derivative with respect to first reference coordinate at
-  // a certain number of sampling points (for element tag 'data._tag')
-  // The number of sampling points is determined by 'data._spaceOrder'.
+  // Matrix dShapeMat_dX, when multiplied by Lagrange coefficients
+  // (i.e. node coordinates), gives the derivative of the mapping
+  // w.r.t. the first reference coordinate at some sampling points.
+  // Those sampling points is determined by 'fsdata'.
   // The ordering of the sampling points is "ordered" (see pointsGenerator.cpp)
   // and is thus different from the Gmsh ordering convention. This is for being
-  // able to convert sampling of jacobian from lagrange to bezier space easily.
+  // able to convert Lagrange coefficients to Bezier coefficients easily.
   fullMatrix<double> samplingPoints;
-  gmshGenerateOrderedPoints(data, samplingPoints);
+  gmshGenerateOrderedPoints(fsdata, samplingPoints);
   const int numSampPnts = samplingPoints.size1();
 
-  // Store shape function gradients of mapping at Jacobian nodes
+  // Store partial derivatives of shape functions at sampling points
   fullMatrix<double> allDPsi;
   const nodalBasis *mapBasis = BasisFactory::getNodalBasis(_elementTag);
   mapBasis->df(samplingPoints, allDPsi);
   const int numMapNodes = allDPsi.size2();
 
-  gradShapeMatX.resize(numSampPnts, numMapNodes);
-  gradShapeMatY.resize(numSampPnts, numMapNodes);
-  gradShapeMatZ.resize(numSampPnts, numMapNodes);
+  dShapeMat_dX.resize(numSampPnts, numMapNodes);
+  dShapeMat_dY.resize(numSampPnts, numMapNodes);
+  dShapeMat_dZ.resize(numSampPnts, numMapNodes);
   for(int i = 0; i < numSampPnts; i++) {
     for(int j = 0; j < numMapNodes; j++) {
-      gradShapeMatX(i, j) = allDPsi(3 * i + 0, j);
-      gradShapeMatY(i, j) = allDPsi(3 * i + 1, j);
-      gradShapeMatZ(i, j) = allDPsi(3 * i + 2, j);
+      dShapeMat_dX(i, j) = allDPsi(3 * i + 0, j);
+      dShapeMat_dY(i, j) = allDPsi(3 * i + 1, j);
+      dShapeMat_dZ(i, j) = allDPsi(3 * i + 2, j);
     }
   }
 
-  gradShapeIdealMatX = gradShapeMatX;
-  gradShapeIdealMatY = gradShapeMatY;
-  gradShapeIdealMatZ = gradShapeMatZ;
-  mapFromIdealElement(_data.getType(), gradShapeIdealMatX, gradShapeIdealMatY,
-                      gradShapeIdealMatZ);
+  dShapeIdealMat_dX = dShapeMat_dX;
+  dShapeIdealMat_dY = dShapeMat_dY;
+  dShapeIdealMat_dZ = dShapeMat_dZ;
+  mapFromIdealElement(_data.getType(), dShapeIdealMat_dX, dShapeIdealMat_dY,
+                      dShapeIdealMat_dZ);
 }
 
-void GradientBasis::getGradientsFromNodes(const fullMatrix<double> &nodes,
+void GradientBasis::getIdealGradientsFromNodes(
+  const fullMatrix<double> &nodesCoord, fullMatrix<double> *dxyzdX,
+  fullMatrix<double> *dxyzdY, fullMatrix<double> *dxyzdZ) const
+{
+  if(dxyzdX) dShapeIdealMat_dX.mult(nodesCoord, *dxyzdX);
+  if(dxyzdY) dShapeIdealMat_dY.mult(nodesCoord, *dxyzdY);
+  if(dxyzdZ) dShapeIdealMat_dZ.mult(nodesCoord, *dxyzdZ);
+}
+
+void GradientBasis::getGradientsFromNodes(const fullMatrix<double> &nodesCoord,
                                           fullMatrix<double> *dxyzdX,
                                           fullMatrix<double> *dxyzdY,
                                           fullMatrix<double> *dxyzdZ) const
 {
-  if(dxyzdX) gradShapeMatX.mult(nodes, *dxyzdX);
-  if(dxyzdY) gradShapeMatY.mult(nodes, *dxyzdY);
-  if(dxyzdZ) gradShapeMatZ.mult(nodes, *dxyzdZ);
+  if(dxyzdX) dShapeMat_dX.mult(nodesCoord, *dxyzdX);
+  if(dxyzdY) dShapeMat_dY.mult(nodesCoord, *dxyzdY);
+  if(dxyzdZ) dShapeMat_dZ.mult(nodesCoord, *dxyzdZ);
 }
 
-void GradientBasis::getAllGradientsFromNodes(const fullMatrix<double> &nodes,
-                                             fullMatrix<double> &dxyzdXYZ) const
+void GradientBasis::getAllGradientsFromNodes(
+  const fullMatrix<double> &nodesCoord, fullMatrix<double> &dxyzdXYZ) const
 {
   fullMatrix<double> prox;
   prox.setAsProxy(dxyzdXYZ, 0, 3);
-  gradShapeMatX.mult(nodes, prox);
+  dShapeMat_dX.mult(nodesCoord, prox);
 
   prox.setAsProxy(dxyzdXYZ, 3, 3);
-  gradShapeMatY.mult(nodes, prox);
+  dShapeMat_dY.mult(nodesCoord, prox);
 
   prox.setAsProxy(dxyzdXYZ, 6, 3);
-  gradShapeMatZ.mult(nodes, prox);
+  dShapeMat_dZ.mult(nodesCoord, prox);
 }
 
 void GradientBasis::getAllIdealGradientsFromNodes(
-  const fullMatrix<double> &nodes, fullMatrix<double> &dxyzdXYZ) const
+  const fullMatrix<double> &nodesCoord, fullMatrix<double> &dxyzdXYZ) const
 {
   fullMatrix<double> prox;
   prox.setAsProxy(dxyzdXYZ, 0, 3);
-  gradShapeIdealMatX.mult(nodes, prox);
+  dShapeIdealMat_dX.mult(nodesCoord, prox);
 
   prox.setAsProxy(dxyzdXYZ, 3, 3);
-  gradShapeIdealMatY.mult(nodes, prox);
+  dShapeIdealMat_dY.mult(nodesCoord, prox);
 
   prox.setAsProxy(dxyzdXYZ, 6, 3);
-  gradShapeIdealMatZ.mult(nodes, prox);
+  dShapeIdealMat_dZ.mult(nodesCoord, prox);
 }
 
-void GradientBasis::getIdealGradientsFromNodes(const fullMatrix<double> &nodes,
-                                               fullMatrix<double> *dxyzdX,
-                                               fullMatrix<double> *dxyzdY,
-                                               fullMatrix<double> *dxyzdZ) const
+void GradientBasis::mapFromIdealElement(int type, fullMatrix<double> &dSMat_dX,
+                                        fullMatrix<double> &dSMat_dY,
+                                        fullMatrix<double> &dSMat_dZ)
 {
-  if(dxyzdX) gradShapeIdealMatX.mult(nodes, *dxyzdX);
-  if(dxyzdY) gradShapeIdealMatY.mult(nodes, *dxyzdY);
-  if(dxyzdZ) gradShapeIdealMatZ.mult(nodes, *dxyzdZ);
+  calcMapFromIdealElement(type, dSMat_dX, dSMat_dY, dSMat_dZ);
 }
 
-void GradientBasis::mapFromIdealElement(int type, fullMatrix<double> &gSMatX,
-                                        fullMatrix<double> &gSMatY,
-                                        fullMatrix<double> &gSMatZ)
+void GradientBasis::mapFromIdealElement(int type, fullVector<double> &dSVec_dX,
+                                        fullVector<double> &dSVec_dY,
+                                        fullVector<double> &dSVec_dZ)
 {
-  calcMapFromIdealElement(type, gSMatX, gSMatY, gSMatZ);
-}
-
-void GradientBasis::mapFromIdealElement(int type, fullVector<double> &gSVecX,
-                                        fullVector<double> &gSVecY,
-                                        fullVector<double> &gSVecZ)
-{
-  calcMapFromIdealElement(type, gSVecX, gSVecY, gSVecZ);
+  calcMapFromIdealElement(type, dSVec_dX, dSVec_dY, dSVec_dZ);
 }
 
 void GradientBasis::mapFromIdealElement(int type, double jac[3][3])
@@ -252,7 +257,7 @@ JacobianBasis::JacobianBasis(int elementTag, FuncSpaceData data)
 
   fullMatrix<double> samplingPoints;
   gmshGeneratePoints(data, samplingPoints);
-  numJacNodes = samplingPoints.size1();
+  numSamplingPnts = samplingPoints.size1();
 
   // Store shape function gradients of mapping at Jacobian nodes
   _gradBasis = BasisFactory::getGradientBasis(elementTag, data);
@@ -260,9 +265,9 @@ JacobianBasis::JacobianBasis(int elementTag, FuncSpaceData data)
   // Compute matrix for lifting from primary Jacobian basis to Jacobian basis
   int primJacType = ElementType::getType(parentType, primJacobianOrder, false);
   const nodalBasis *primJacBasis = BasisFactory::getNodalBasis(primJacType);
-  numPrimJacNodes = primJacBasis->getNumShapeFunctions();
+  numPrimSamplingPnts = primJacBasis->getNumShapeFunctions();
 
-  matrixPrimJac2Jac.resize(numJacNodes, numPrimJacNodes);
+  matrixPrimJac2Jac.resize(numSamplingPnts, numPrimSamplingPnts);
   primJacBasis->f(samplingPoints, matrixPrimJac2Jac);
 
   // Compute shape function gradients of primary mapping at barycenter, in order
@@ -285,27 +290,27 @@ JacobianBasis::JacobianBasis(int elementTag, FuncSpaceData data)
   double(*barDPsi)[3] = new double[numPrimMapNodes][3];
   primMapBasis->df(xBar, yBar, zBar, barDPsi);
 
-  primGradShapeBaryX.resize(numPrimMapNodes);
-  primGradShapeBaryY.resize(numPrimMapNodes);
-  primGradShapeBaryZ.resize(numPrimMapNodes);
+  dPrimBaryShape_dX.resize(numPrimMapNodes);
+  dPrimBaryShape_dY.resize(numPrimMapNodes);
+  dPrimBaryShape_dZ.resize(numPrimMapNodes);
   for(int j = 0; j < numPrimMapNodes; j++) {
-    primGradShapeBaryX(j) = barDPsi[j][0];
-    primGradShapeBaryY(j) = barDPsi[j][1];
-    primGradShapeBaryZ(j) = barDPsi[j][2];
+    dPrimBaryShape_dX(j) = barDPsi[j][0];
+    dPrimBaryShape_dY(j) = barDPsi[j][1];
+    dPrimBaryShape_dZ(j) = barDPsi[j][2];
   }
 
-  primIdealGradShapeBaryX = primGradShapeBaryX;
-  primIdealGradShapeBaryY = primGradShapeBaryY;
-  primIdealGradShapeBaryZ = primGradShapeBaryZ;
+  dPrimBaryIdealShape_dX = dPrimBaryShape_dX;
+  dPrimBaryIdealShape_dY = dPrimBaryShape_dY;
+  dPrimBaryIdealShape_dZ = dPrimBaryShape_dZ;
   _gradBasis->mapFromIdealElement(
-    primIdealGradShapeBaryX, primIdealGradShapeBaryY, primIdealGradShapeBaryZ);
+    dPrimBaryIdealShape_dX, dPrimBaryIdealShape_dY, dPrimBaryIdealShape_dZ);
 
   delete[] barDPsi;
 
   // Compute "fast" Jacobian evaluation matrices (at 1st order nodes +
   // barycenter)
-  numJacNodesFast = numPrimMapNodes + 1;
-  fullMatrix<double> lagPointsFast(numJacNodesFast, 3); // Sampling points
+  numSamplingPntsFast = numPrimMapNodes + 1;
+  fullMatrix<double> lagPointsFast(numSamplingPntsFast, 3); // Sampling points
   lagPointsFast.copy(primMapBasis->points, 0, numPrimMapNodes, 0,
                      primMapBasis->points.size2(), 0, 0); // 1st order nodes
   lagPointsFast(numPrimMapNodes, 0) = barycenter[0]; // Last point = barycenter
@@ -317,14 +322,14 @@ JacobianBasis::JacobianBasis(int elementTag, FuncSpaceData data)
   mapBasis->df(lagPointsFast, allDPsiFast);
   numMapNodes = mapBasis->getNumShapeFunctions();
 
-  gradShapeMatXFast.resize(numJacNodesFast, numMapNodes);
-  gradShapeMatYFast.resize(numJacNodesFast, numMapNodes);
-  gradShapeMatZFast.resize(numJacNodesFast, numMapNodes);
-  for(int i = 0; i < numJacNodesFast; i++) {
+  dFastShapeMat_dX.resize(numSamplingPntsFast, numMapNodes);
+  dFastShapeMat_dY.resize(numSamplingPntsFast, numMapNodes);
+  dFastShapeMat_dZ.resize(numSamplingPntsFast, numMapNodes);
+  for(int i = 0; i < numSamplingPntsFast; i++) {
     for(int j = 0; j < numMapNodes; j++) {
-      gradShapeMatXFast(i, j) = allDPsiFast(3 * i + 0, j);
-      gradShapeMatYFast(i, j) = allDPsiFast(3 * i + 1, j);
-      gradShapeMatZFast(i, j) = allDPsiFast(3 * i + 2, j);
+      dFastShapeMat_dX(i, j) = allDPsiFast(3 * i + 0, j);
+      dFastShapeMat_dY(i, j) = allDPsiFast(3 * i + 1, j);
+      dFastShapeMat_dZ(i, j) = allDPsiFast(3 * i + 2, j);
     }
   }
 }
@@ -336,9 +341,9 @@ double JacobianBasis::getPrimNormals1D(const fullMatrix<double> &nodesXYZ,
 {
   fullVector<double> dxyzdXbar(3);
   for(int j = 0; j < numPrimMapNodes; j++) {
-    dxyzdXbar(0) += primGradShapeBaryX(j) * nodesXYZ(j, 0);
-    dxyzdXbar(1) += primGradShapeBaryX(j) * nodesXYZ(j, 1);
-    dxyzdXbar(2) += primGradShapeBaryX(j) * nodesXYZ(j, 2);
+    dxyzdXbar(0) += dPrimBaryShape_dX(j) * nodesXYZ(j, 0);
+    dxyzdXbar(1) += dPrimBaryShape_dX(j) * nodesXYZ(j, 1);
+    dxyzdXbar(2) += dPrimBaryShape_dX(j) * nodesXYZ(j, 2);
   }
 
   if((fabs(dxyzdXbar(0)) >= fabs(dxyzdXbar(1)) &&
@@ -382,9 +387,9 @@ double JacobianBasis::getPrimNormal2D(const fullMatrix<double> &nodesXYZ,
                                       bool ideal) const
 {
   const fullVector<double> &gSX =
-    ideal ? primIdealGradShapeBaryX : primGradShapeBaryX;
+    ideal ? dPrimBaryIdealShape_dX : dPrimBaryShape_dX;
   const fullVector<double> &gSY =
-    ideal ? primIdealGradShapeBaryY : primGradShapeBaryY;
+    ideal ? dPrimBaryIdealShape_dY : dPrimBaryShape_dY;
   fullMatrix<double> dxyzdX(1, 3), dxyzdY(1, 3);
   for(int j = 0; j < numPrimMapNodes; j++) {
     dxyzdX(0, 0) += gSX(j) * nodesXYZ(j, 0);
@@ -414,11 +419,11 @@ double JacobianBasis::getPrimJac3D(const fullMatrix<double> &nodesXYZ,
                                    bool ideal) const
 {
   const fullVector<double> &gSX =
-    ideal ? primIdealGradShapeBaryX : primGradShapeBaryX;
+    ideal ? dPrimBaryIdealShape_dX : dPrimBaryShape_dX;
   const fullVector<double> &gSY =
-    ideal ? primIdealGradShapeBaryY : primGradShapeBaryY;
+    ideal ? dPrimBaryIdealShape_dY : dPrimBaryShape_dY;
   const fullVector<double> &gSZ =
-    ideal ? primIdealGradShapeBaryZ : primGradShapeBaryZ;
+    ideal ? dPrimBaryIdealShape_dZ : dPrimBaryShape_dZ;
   fullMatrix<double> dxyzdX(1, 3), dxyzdY(1, 3), dxyzdZ(1, 3);
   for(int j = 0; j < numPrimMapNodes; j++) {
     dxyzdX(0, 0) += gSX(j) * nodesXYZ(j, 0);
@@ -441,14 +446,14 @@ double JacobianBasis::getPrimJac3D(const fullMatrix<double> &nodesXYZ,
 // vectors to straight element for regularization. Evaluation points depend on
 // the given matrices for shape function gradients.
 void JacobianBasis::getJacobianGeneral(
-  int nJacNodes, const fullMatrix<double> &gSMatX,
-  const fullMatrix<double> &gSMatY, const fullMatrix<double> &gSMatZ,
+  int nSamplingPnts, const fullMatrix<double> &dSMat_dX,
+  const fullMatrix<double> &dSMat_dY, const fullMatrix<double> &dSMat_dZ,
   const fullMatrix<double> &nodesXYZ, bool idealNorm, bool scaling,
   fullVector<double> &jacobian, const fullMatrix<double> *geomNormals) const
 {
   switch(_dim) {
   case 0: {
-    for(int i = 0; i < nJacNodes; i++) jacobian(i) = 1.;
+    for(int i = 0; i < nSamplingPnts; i++) jacobian(i) = 1.;
   } break;
   case 1: {
     fullMatrix<double> normals(2, 3);
@@ -456,7 +461,7 @@ void JacobianBasis::getJacobianGeneral(
     if(geomNormals) normals.setAll(*geomNormals);
     if(scaling) {
       if(invScale == 0) {
-        for(int i = 0; i < nJacNodes; i++) jacobian(i) = 0;
+        for(int i = 0; i < nSamplingPnts; i++) jacobian(i) = 0;
         return;
       }
       const double scale = 1. / invScale;
@@ -465,9 +470,9 @@ void JacobianBasis::getJacobianGeneral(
       normals(0, 1) *= scale;
       normals(0, 2) *= scale;
     }
-    fullMatrix<double> dxyzdX(nJacNodes, 3);
-    gSMatX.mult(nodesXYZ, dxyzdX);
-    for(int i = 0; i < nJacNodes; i++) {
+    fullMatrix<double> dxyzdX(nSamplingPnts, 3);
+    dSMat_dX.mult(nodesXYZ, dxyzdX);
+    for(int i = 0; i < nSamplingPnts; i++) {
       const double &dxdX = dxyzdX(i, 0), &dydX = dxyzdX(i, 1),
                    &dzdX = dxyzdX(i, 2);
       const double &dxdY = normals(0, 0), &dydY = normals(0, 1),
@@ -484,7 +489,7 @@ void JacobianBasis::getJacobianGeneral(
     if(geomNormals) normal.setAll(*geomNormals);
     if(scaling) {
       if(invScale == 0) {
-        for(int i = 0; i < nJacNodes; i++) jacobian(i) = 0;
+        for(int i = 0; i < nSamplingPnts; i++) jacobian(i) = 0;
         return;
       }
       const double scale = 1. / invScale;
@@ -493,10 +498,10 @@ void JacobianBasis::getJacobianGeneral(
       normal(0, 1) *= scale;
       normal(0, 2) *= scale;
     }
-    fullMatrix<double> dxyzdX(nJacNodes, 3), dxyzdY(nJacNodes, 3);
-    gSMatX.mult(nodesXYZ, dxyzdX);
-    gSMatY.mult(nodesXYZ, dxyzdY);
-    for(int i = 0; i < nJacNodes; i++) {
+    fullMatrix<double> dxyzdX(nSamplingPnts, 3), dxyzdY(nSamplingPnts, 3);
+    dSMat_dX.mult(nodesXYZ, dxyzdX);
+    dSMat_dY.mult(nodesXYZ, dxyzdY);
+    for(int i = 0; i < nSamplingPnts; i++) {
       const double &dxdX = dxyzdX(i, 0), &dydX = dxyzdX(i, 1),
                    &dzdX = dxyzdX(i, 2);
       const double &dxdY = dxyzdY(i, 0), &dydY = dxyzdY(i, 1),
@@ -509,12 +514,12 @@ void JacobianBasis::getJacobianGeneral(
   } break;
   case 3: {
     fullMatrix<double> dum;
-    fullMatrix<double> dxyzdX(nJacNodes, 3), dxyzdY(nJacNodes, 3),
-      dxyzdZ(nJacNodes, 3);
-    gSMatX.mult(nodesXYZ, dxyzdX);
-    gSMatY.mult(nodesXYZ, dxyzdY);
-    gSMatZ.mult(nodesXYZ, dxyzdZ);
-    for(int i = 0; i < nJacNodes; i++) {
+    fullMatrix<double> dxyzdX(nSamplingPnts, 3), dxyzdY(nSamplingPnts, 3),
+      dxyzdZ(nSamplingPnts, 3);
+    dSMat_dX.mult(nodesXYZ, dxyzdX);
+    dSMat_dY.mult(nodesXYZ, dxyzdY);
+    dSMat_dZ.mult(nodesXYZ, dxyzdZ);
+    for(int i = 0; i < nSamplingPnts; i++) {
       const double &dxdX = dxyzdX(i, 0), &dydX = dxyzdX(i, 1),
                    &dzdX = dxyzdX(i, 2);
       const double &dxdY = dxyzdY(i, 0), &dydY = dxyzdY(i, 1),
@@ -537,8 +542,8 @@ void JacobianBasis::getJacobianGeneral(
 // depend on the given matrices for shape function gradients.  TODO: Optimize
 // and test 1D & 2D
 void JacobianBasis::getJacobianGeneral(
-  int nJacNodes, const fullMatrix<double> &gSMatX,
-  const fullMatrix<double> &gSMatY, const fullMatrix<double> &gSMatZ,
+  int nSamplingPnts, const fullMatrix<double> &dSMat_dX,
+  const fullMatrix<double> &dSMat_dY, const fullMatrix<double> &dSMat_dZ,
   const fullMatrix<double> &nodesX, const fullMatrix<double> &nodesY,
   const fullMatrix<double> &nodesZ, bool idealNorm, bool scaling,
   fullMatrix<double> &jacobian, const fullMatrix<double> *geomNormals) const
@@ -547,16 +552,16 @@ void JacobianBasis::getJacobianGeneral(
   case 0: {
     const int numEl = nodesX.size2();
     for(int iEl = 0; iEl < numEl; iEl++)
-      for(int i = 0; i < nJacNodes; i++) jacobian(i, iEl) = 1.;
+      for(int i = 0; i < nSamplingPnts; i++) jacobian(i, iEl) = 1.;
   } break;
   case 1: {
     const int numEl = nodesX.size2();
-    fullMatrix<double> dxdX(nJacNodes, numEl);
-    fullMatrix<double> dydX(nJacNodes, numEl);
-    fullMatrix<double> dzdX(nJacNodes, numEl);
-    gSMatX.mult(nodesX, dxdX);
-    gSMatX.mult(nodesY, dydX);
-    gSMatX.mult(nodesZ, dzdX);
+    fullMatrix<double> dxdX(nSamplingPnts, numEl);
+    fullMatrix<double> dydX(nSamplingPnts, numEl);
+    fullMatrix<double> dzdX(nSamplingPnts, numEl);
+    dSMat_dX.mult(nodesX, dxdX);
+    dSMat_dX.mult(nodesY, dydX);
+    dSMat_dX.mult(nodesZ, dzdX);
     for(int iEl = 0; iEl < numEl; iEl++) {
       fullMatrix<double> nodesXYZ(numPrimMapNodes, 3);
       for(int i = 0; i < numPrimMapNodes; i++) {
@@ -569,7 +574,7 @@ void JacobianBasis::getJacobianGeneral(
       if(geomNormals) normals.setAll(*geomNormals);
       if(scaling) {
         if(invScale == 0) {
-          for(int i = 0; i < nJacNodes; i++) jacobian(i, iEl) = 0;
+          for(int i = 0; i < nSamplingPnts; i++) jacobian(i, iEl) = 0;
           continue;
         }
         const double scale = 1. / invScale;
@@ -582,25 +587,25 @@ void JacobianBasis::getJacobianGeneral(
                    &dzdY = normals(0, 2);
       const double &dxdZ = normals(1, 0), &dydZ = normals(1, 1),
                    &dzdZ = normals(1, 2);
-      for(int i = 0; i < nJacNodes; i++)
+      for(int i = 0; i < nSamplingPnts; i++)
         jacobian(i, iEl) = calcDet3D(dxdX(i, iEl), dxdY, dxdZ, dydX(i, iEl),
                                      dydY, dydZ, dzdX(i, iEl), dzdY, dzdZ);
     }
   } break;
   case 2: {
     const int numEl = nodesX.size2();
-    fullMatrix<double> dxdX(nJacNodes, numEl);
-    fullMatrix<double> dydX(nJacNodes, numEl);
-    fullMatrix<double> dzdX(nJacNodes, numEl);
-    fullMatrix<double> dxdY(nJacNodes, numEl);
-    fullMatrix<double> dydY(nJacNodes, numEl);
-    fullMatrix<double> dzdY(nJacNodes, numEl);
-    gSMatX.mult(nodesX, dxdX);
-    gSMatX.mult(nodesY, dydX);
-    gSMatX.mult(nodesZ, dzdX);
-    gSMatY.mult(nodesX, dxdY);
-    gSMatY.mult(nodesY, dydY);
-    gSMatY.mult(nodesZ, dzdY);
+    fullMatrix<double> dxdX(nSamplingPnts, numEl);
+    fullMatrix<double> dydX(nSamplingPnts, numEl);
+    fullMatrix<double> dzdX(nSamplingPnts, numEl);
+    fullMatrix<double> dxdY(nSamplingPnts, numEl);
+    fullMatrix<double> dydY(nSamplingPnts, numEl);
+    fullMatrix<double> dzdY(nSamplingPnts, numEl);
+    dSMat_dX.mult(nodesX, dxdX);
+    dSMat_dX.mult(nodesY, dydX);
+    dSMat_dX.mult(nodesZ, dzdX);
+    dSMat_dY.mult(nodesX, dxdY);
+    dSMat_dY.mult(nodesY, dydY);
+    dSMat_dY.mult(nodesZ, dzdY);
     for(int iEl = 0; iEl < numEl; iEl++) {
       fullMatrix<double> nodesXYZ(numPrimMapNodes, 3);
       for(int i = 0; i < numPrimMapNodes; i++) {
@@ -613,7 +618,7 @@ void JacobianBasis::getJacobianGeneral(
       if(geomNormals) normal.setAll(*geomNormals);
       if(scaling) {
         if(invScale == 0) {
-          for(int i = 0; i < nJacNodes; i++) jacobian(i, iEl) = 0;
+          for(int i = 0; i < nSamplingPnts; i++) jacobian(i, iEl) = 0;
           continue;
         }
         const double scale = 1. / invScale;
@@ -624,7 +629,7 @@ void JacobianBasis::getJacobianGeneral(
       }
       const double &dxdZ = normal(0, 0), &dydZ = normal(0, 1),
                    &dzdZ = normal(0, 2);
-      for(int i = 0; i < nJacNodes; i++)
+      for(int i = 0; i < nSamplingPnts; i++)
         jacobian(i, iEl) =
           calcDet3D(dxdX(i, iEl), dxdY(i, iEl), dxdZ, dydX(i, iEl),
                     dydY(i, iEl), dydZ, dzdX(i, iEl), dzdY(i, iEl), dzdZ);
@@ -632,26 +637,26 @@ void JacobianBasis::getJacobianGeneral(
   } break;
   case 3: {
     const int numEl = nodesX.size2();
-    fullMatrix<double> dxdX(nJacNodes, numEl);
-    fullMatrix<double> dydX(nJacNodes, numEl);
-    fullMatrix<double> dzdX(nJacNodes, numEl);
-    fullMatrix<double> dxdY(nJacNodes, numEl);
-    fullMatrix<double> dydY(nJacNodes, numEl);
-    fullMatrix<double> dzdY(nJacNodes, numEl);
-    fullMatrix<double> dxdZ(nJacNodes, numEl);
-    fullMatrix<double> dydZ(nJacNodes, numEl);
-    fullMatrix<double> dzdZ(nJacNodes, numEl);
-    gSMatX.mult(nodesX, dxdX);
-    gSMatX.mult(nodesY, dydX);
-    gSMatX.mult(nodesZ, dzdX);
-    gSMatY.mult(nodesX, dxdY);
-    gSMatY.mult(nodesY, dydY);
-    gSMatY.mult(nodesZ, dzdY);
-    gSMatZ.mult(nodesX, dxdZ);
-    gSMatZ.mult(nodesY, dydZ);
-    gSMatZ.mult(nodesZ, dzdZ);
+    fullMatrix<double> dxdX(nSamplingPnts, numEl);
+    fullMatrix<double> dydX(nSamplingPnts, numEl);
+    fullMatrix<double> dzdX(nSamplingPnts, numEl);
+    fullMatrix<double> dxdY(nSamplingPnts, numEl);
+    fullMatrix<double> dydY(nSamplingPnts, numEl);
+    fullMatrix<double> dzdY(nSamplingPnts, numEl);
+    fullMatrix<double> dxdZ(nSamplingPnts, numEl);
+    fullMatrix<double> dydZ(nSamplingPnts, numEl);
+    fullMatrix<double> dzdZ(nSamplingPnts, numEl);
+    dSMat_dX.mult(nodesX, dxdX);
+    dSMat_dX.mult(nodesY, dydX);
+    dSMat_dX.mult(nodesZ, dzdX);
+    dSMat_dY.mult(nodesX, dxdY);
+    dSMat_dY.mult(nodesY, dydY);
+    dSMat_dY.mult(nodesZ, dzdY);
+    dSMat_dZ.mult(nodesX, dxdZ);
+    dSMat_dZ.mult(nodesY, dydZ);
+    dSMat_dZ.mult(nodesZ, dzdZ);
     for(int iEl = 0; iEl < numEl; iEl++) {
-      for(int i = 0; i < nJacNodes; i++)
+      for(int i = 0; i < nSamplingPnts; i++)
         jacobian(i, iEl) = calcDet3D(dxdX(i, iEl), dxdY(i, iEl), dxdZ(i, iEl),
                                      dydX(i, iEl), dydY(i, iEl), dydZ(i, iEl),
                                      dzdX(i, iEl), dzdY(i, iEl), dzdZ(i, iEl));
@@ -663,25 +668,38 @@ void JacobianBasis::getJacobianGeneral(
           nodesXYZ(i, 2) = nodesZ(i, iEl);
         }
         const double scale = 1. / getPrimJac3D(nodesXYZ);
-        for(int i = 0; i < nJacNodes; i++) jacobian(i, iEl) *= scale;
+        for(int i = 0; i < nSamplingPnts; i++) jacobian(i, iEl) *= scale;
       }
     }
   } break;
   }
 }
 
-// Calculate (signed) Jacobian and its gradients for one element, with normal
-// vectors to straight element for regularization. Evaluation points depend on
-// the given matrices for shape function gradients.
+// Calculate the (signed) Jacobian determinant (in short, J) and its partial
+// derivatives w.r.t. nodes coordinates for the element defined by
+// the given node positions, with given normal vectors to straight element
+// for regularization of 1D and 2D elements).
+// Sampling points depend on the input matrices of shape function partial
+// derivatives 'dSMat_d*', and only the 'nSamplingPnts' first of them
+// are computed.
+// The result is written in the matrix 'JDJ' which should be of size at
+// least "nSamplingPnts x (3 * numMapNodes + 1)".
+// For each sampling point, a row of 'JDJ' is filled with:
+// - the partial derivatives of J w.r.t. the x component of the nodes
+// - the partial derivatives of J w.r.t. the y component of the nodes
+// - the partial derivatives of J w.r.t. the z component of the nodes
+// - J
+// NB: (x, y, z) are the physical coordinates and (X, Y, Z) are the reference
+// coordinates
 void JacobianBasis::getSignedJacAndGradientsGeneral(
-  int nJacNodes, const fullMatrix<double> &gSMatX,
-  const fullMatrix<double> &gSMatY, const fullMatrix<double> &gSMatZ,
+  int nSamplingPnts, const fullMatrix<double> &dSMat_dX,
+  const fullMatrix<double> &dSMat_dY, const fullMatrix<double> &dSMat_dZ,
   const fullMatrix<double> &nodesXYZ, const fullMatrix<double> &normals,
   fullMatrix<double> &JDJ) const
 {
   switch(_dim) {
   case 0: {
-    for(int i = 0; i < nJacNodes; i++) {
+    for(int i = 0; i < nSamplingPnts; i++) {
       for(int j = 0; j < numMapNodes; j++) {
         JDJ(i, j) = 0.;
         JDJ(i, j + 1 * numMapNodes) = 0.;
@@ -691,62 +709,52 @@ void JacobianBasis::getSignedJacAndGradientsGeneral(
     }
   } break;
   case 1: {
-    fullMatrix<double> dxyzdX(nJacNodes, 3), dxyzdY(nJacNodes, 3);
-    gSMatX.mult(nodesXYZ, dxyzdX);
-    for(int i = 0; i < nJacNodes; i++) {
-      const double &dxdX = dxyzdX(i, 0), &dydX = dxyzdX(i, 1),
-                   &dzdX = dxyzdX(i, 2);
-      const double &dxdY = normals(0, 0), &dydY = normals(0, 1),
-                   &dzdY = normals(0, 2);
-      const double &dxdZ = normals(1, 0), &dydZ = normals(1, 1),
-                   &dzdZ = normals(1, 2);
-      calcJDJ1D(dxdX, dxdY, dxdZ, dydX, dydY, dydZ, dzdX, dzdY, dzdZ, i,
-                numMapNodes, gSMatX, JDJ);
+    fullMatrix<double> dxyzdX(nSamplingPnts, 3);
+    dSMat_dX.mult(nodesXYZ, dxyzdX);
+    for(int i = 0; i < nSamplingPnts; i++) {
+      calcJDJ1D(dxyzdX(i, 0), normals(0, 0), normals(1, 0),
+                dxyzdX(i, 1), normals(0, 1), normals(1, 1),
+                dxyzdX(i, 2), normals(0, 2), normals(1, 2),
+                i, numMapNodes, dSMat_dX, JDJ);
     }
   } break;
   case 2: {
-    fullMatrix<double> dxyzdX(nJacNodes, 3), dxyzdY(nJacNodes, 3);
-    gSMatX.mult(nodesXYZ, dxyzdX);
-    gSMatY.mult(nodesXYZ, dxyzdY);
-    for(int i = 0; i < nJacNodes; i++) {
-      const double &dxdX = dxyzdX(i, 0), &dydX = dxyzdX(i, 1),
-                   &dzdX = dxyzdX(i, 2);
-      const double &dxdY = dxyzdY(i, 0), &dydY = dxyzdY(i, 1),
-                   &dzdY = dxyzdY(i, 2);
-      const double &dxdZ = normals(0, 0), &dydZ = normals(0, 1),
-                   &dzdZ = normals(0, 2);
-      calcJDJ2D(dxdX, dxdY, dxdZ, dydX, dydY, dydZ, dzdX, dzdY, dzdZ, i,
-                numMapNodes, gSMatX, gSMatY, JDJ);
+    fullMatrix<double> dxyzdX(nSamplingPnts, 3);
+    fullMatrix<double> dxyzdY(nSamplingPnts, 3);
+    dSMat_dX.mult(nodesXYZ, dxyzdX);
+    dSMat_dY.mult(nodesXYZ, dxyzdY);
+    for(int i = 0; i < nSamplingPnts; i++) {
+      calcJDJ2D(dxyzdX(i, 0), dxyzdY(i, 0), normals(0, 0),
+                dxyzdX(i, 1), dxyzdY(i, 1), normals(0, 1),
+                dxyzdX(i, 2), dxyzdY(i, 2), normals(0, 2),
+                i, numMapNodes, dSMat_dX, dSMat_dY, JDJ);
     }
   } break;
   case 3: {
-    fullMatrix<double> dxyzdX(nJacNodes, 3), dxyzdY(nJacNodes, 3),
-      dxyzdZ(nJacNodes, 3);
-    gSMatX.mult(nodesXYZ, dxyzdX);
-    gSMatY.mult(nodesXYZ, dxyzdY);
-    gSMatZ.mult(nodesXYZ, dxyzdZ);
-    for(int i = 0; i < nJacNodes; i++) {
-      const double &dxdX = dxyzdX(i, 0), &dydX = dxyzdX(i, 1),
-                   &dzdX = dxyzdX(i, 2);
-      const double &dxdY = dxyzdY(i, 0), &dydY = dxyzdY(i, 1),
-                   &dzdY = dxyzdY(i, 2);
-      const double &dxdZ = dxyzdZ(i, 0), &dydZ = dxyzdZ(i, 1),
-                   &dzdZ = dxyzdZ(i, 2);
-      calcJDJ3D(dxdX, dxdY, dxdZ, dydX, dydY, dydZ, dzdX, dzdY, dzdZ, i,
-                numMapNodes, gSMatX, gSMatY, gSMatZ, JDJ);
+    fullMatrix<double> dxyzdX(nSamplingPnts, 3);
+    fullMatrix<double> dxyzdY(nSamplingPnts, 3);
+    fullMatrix<double> dxyzdZ(nSamplingPnts, 3);
+    dSMat_dX.mult(nodesXYZ, dxyzdX);
+    dSMat_dY.mult(nodesXYZ, dxyzdY);
+    dSMat_dZ.mult(nodesXYZ, dxyzdZ);
+    for(int i = 0; i < nSamplingPnts; i++) {
+      calcJDJ3D(dxyzdX(i, 0), dxyzdY(i, 0), dxyzdZ(i, 0),
+                dxyzdX(i, 1), dxyzdY(i, 1), dxyzdZ(i, 1),
+                dxyzdX(i, 2), dxyzdY(i, 2), dxyzdZ(i, 2),
+                i, numMapNodes, dSMat_dX, dSMat_dY, dSMat_dZ, JDJ);
     }
   } break;
   }
 }
 
 void JacobianBasis::getSignedIdealJacAndGradientsGeneral(
-  int nJacNodes, const fullMatrix<double> &gSMatX,
-  const fullMatrix<double> &gSMatY, const fullMatrix<double> &gSMatZ,
+  int nSamplingPnts, const fullMatrix<double> &dSMat_dX,
+  const fullMatrix<double> &dSMat_dY, const fullMatrix<double> &dSMat_dZ,
   const fullMatrix<double> &nodesXYZ, const fullMatrix<double> &normals,
   fullMatrix<double> &JDJ) const
 {
-  getSignedJacAndGradientsGeneral(nJacNodes, gSMatX, gSMatY, gSMatZ, nodesXYZ,
-                                  normals, JDJ);
+  getSignedJacAndGradientsGeneral(nSamplingPnts, dSMat_dX, dSMat_dY, dSMat_dZ,
+                                  nodesXYZ, normals, JDJ);
 }
 
 void JacobianBasis::getMetricMinAndGradients(
@@ -765,11 +773,11 @@ void JacobianBasis::getMetricMinAndGradients(
   double invJaci[2][2];
   inv2x2(jaci, invJaci);
 
-  for(int l = 0; l < numJacNodes; l++) {
+  for(int l = 0; l < numSamplingPnts; l++) {
     double jac[2][2] = {{0., 0.}, {0., 0.}};
     for(int i = 0; i < numMapNodes; i++) {
-      const double &dPhidX = _gradBasis->gradShapeMatX(l, i);
-      const double &dPhidY = _gradBasis->gradShapeMatY(l, i);
+      const double &dPhidX = _gradBasis->dShapeMat_dX(l, i);
+      const double &dPhidY = _gradBasis->dShapeMat_dY(l, i);
       const double dpsidx = dPhidX * invJaci[0][0] + dPhidY * invJaci[1][0];
       const double dpsidy = dPhidX * invJaci[0][1] + dPhidY * invJaci[1][1];
       jac[0][0] += nodesXYZ(i, 0) * dpsidx;
@@ -796,8 +804,8 @@ void JacobianBasis::getMetricMinAndGradients(
     const double aetaxi = ayx * invJaci[0][0] + ayy * invJaci[0][1];
     const double axieta = axx * invJaci[1][0] + axy * invJaci[1][1];
     for(int i = 0; i < numMapNodes; i++) {
-      const double &dPhidX = _gradBasis->gradShapeMatX(l, i);
-      const double &dPhidY = _gradBasis->gradShapeMatY(l, i);
+      const double &dPhidX = _gradBasis->dShapeMat_dX(l, i);
+      const double &dPhidY = _gradBasis->dShapeMat_dY(l, i);
       gradLambdaJ(l, i + 0 * numMapNodes) = axixi * dPhidX + axieta * dPhidY;
       gradLambdaJ(l, i + 1 * numMapNodes) = aetaxi * dPhidX + aetaeta * dPhidY;
     }
