@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2019 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2020 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
@@ -8,8 +8,9 @@
 
 #include "Plugin.h"
 
-extern "C" {
-GMSH_Plugin *GMSH_RegisterNewViewPlugin();
+extern "C"
+{
+  GMSH_Plugin *GMSH_RegisterNewViewPlugin();
 }
 
 class GMSH_NewViewPlugin : public GMSH_PostPlugin {
