@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2019 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2020 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // issues on https://gitlab.onelab.info/gmsh/gmsh/issues.
@@ -12,6 +12,8 @@
 #include "StringUtils.h"
 #include "OS.h"
 #include "Context.h"
+#include "CGNSCommon.h"
+#include "CGNSConventions.h"
 
 bool PViewDataGModel::addData(GModel *model,
                               const std::map<int, std::vector<double> > &data,
@@ -55,9 +57,14 @@ bool PViewDataGModel::addData(GModel *model, const std::vector<std::size_t> &tag
   if(data.empty() || tags.empty() || data.size() != tags.size()) return false;
 
   if(numComp < 0) {
-    numComp = 9;
-    for(std::size_t i = 0; i < data.size(); i++)
-      numComp = std::min(numComp, (int)data[i].size());
+    if(_type == ElementNodeData) {
+      numComp = 1; // cannot infer, as we can have different element types
+    }
+    else{
+      numComp = 9;
+      for(std::size_t i = 0; i < data.size(); i++)
+        numComp = std::min(numComp, (int)data[i].size());
+    }
   }
 
   while(step >= (int)_steps.size())
@@ -80,6 +87,45 @@ bool PViewDataGModel::addData(GModel *model, const std::vector<std::size_t> &tag
   return true;
 }
 
+bool PViewDataGModel::addData(GModel *model, const std::vector<std::size_t> &tags,
+                              const std::vector<double> &data,
+                              int step, double time, int partition, int numComp)
+{
+  if(data.empty() || tags.empty()) return false;
+
+  std::size_t stride = data.size() / tags.size();
+  if(stride < 1) return false;
+
+  if(numComp < 0) {
+    if(_type == ElementNodeData) {
+      numComp = 1; // cannot infer, as we can have different element types
+    }
+    else {
+      numComp = (int)stride;
+    }
+  }
+
+  while(step >= (int)_steps.size())
+    _steps.push_back(new stepData<double>(model, numComp));
+  _steps[step]->fillEntities();
+  _steps[step]->computeBoundingBox();
+  _steps[step]->setTime(time);
+
+  int numEnt = (_type == NodeData) ? model->getNumMeshVertices() :
+                                     model->getNumMeshElements();
+  _steps[step]->resizeData(numEnt);
+
+  int mult = stride / numComp;
+  for(std::size_t i = 0; i < tags.size(); i++) {
+    double *d = _steps[step]->getData(tags[i], true, mult);
+    int k = i * stride;
+    for(std::size_t j = 0; j < stride; j++) d[j] = data[k + j];
+  }
+  if(partition >= 0) _steps[step]->getPartitions().insert(partition);
+  finalize();
+  return true;
+}
+
 void PViewDataGModel::destroyData()
 {
   for(std::size_t i = 0; i < _steps.size(); i++) _steps[i]->destroyData();
@@ -113,7 +159,7 @@ bool PViewDataGModel::readMSH(const std::string &viewName,
 
   _steps[step]->resizeData(numEnt);
 
-  Msg::ResetProgressMeter();
+  Msg::StartProgressMeter(numEnt);
   for(int i = 0; i < numEnt; i++) {
     int num;
     if(binary) {
@@ -156,9 +202,9 @@ bool PViewDataGModel::readMSH(const std::string &viewName,
       _min = std::min(_min, val);
       _max = std::max(_max, val);
     }
-    if(numEnt > 100000) Msg::ProgressMeter(i + 1, numEnt, true, "Reading data");
+    if(numEnt > 100000) Msg::ProgressMeter(i + 1, true, "Reading data");
   }
-
+  Msg::StopProgressMeter();
   if(partition >= 0) _steps[step]->getPartitions().insert(partition);
 
   finalize(false, interpolationScheme);
@@ -265,7 +311,7 @@ bool PViewDataGModel::writeMSH(const std::string &fileName, double version,
           if(_steps[step]->getData(i)) {
             MVertex *v = _steps[step]->getModel()->getMeshVertexByTag(i);
             if(!v) {
-              Msg::Error("Unknown vertex %d in data", i);
+              Msg::Error("Unknown node %d in data", i);
               fclose(fp);
               return false;
             }
@@ -928,7 +974,7 @@ bool PViewDataGModel::writeMED(const std::string &fileName)
     if(_steps[0]->getData(i)) {
       MVertex *v = _steps[0]->getModel()->getMeshVertexByTag(i);
       if(!v) {
-        Msg::Error("Unknown vertex %d in data", i);
+        Msg::Error("Unknown node %d in data", i);
         return false;
       }
       profile.push_back(v->getIndex());
@@ -1080,7 +1126,7 @@ void PViewDataGModel::sendToServer(const std::string &name)
     if(_steps[0]->getData(i)) {
       MVertex *v = _steps[0]->getModel()->getMeshVertexByTag(i);
       if(!v) {
-        Msg::Error("Unknown vertex %d in data", i);
+        Msg::Error("Unknown node %d in data", i);
         return;
       }
       int num = v->getNum();
