@@ -6,7 +6,7 @@ import sys
 # modified mesh in the same model.
 
 if len(sys.argv) < 2:
-    print "Usage: " + sys.argv[0] + " file.msh"
+    print("Usage: " + sys.argv[0] + " file.msh")
     exit(0)
 
 gmsh.initialize()
@@ -24,7 +24,8 @@ entities = gmsh.model.getEntities()
 # get the nodes and elements
 for e in entities:
     nodeTags[e], nodeCoords[e], _ = gmsh.model.mesh.getNodes(e[0], e[1])
-    elementTypes[e], elementTags[e], elementNodeTags[e] = gmsh.model.mesh.getElements(e[0], e[1])
+    elementTypes[e], elementTags[e], elementNodeTags[
+        e] = gmsh.model.mesh.getElements(e[0], e[1])
 
 # delete the mesh
 gmsh.model.mesh.clear()
@@ -34,9 +35,11 @@ for e in entities:
     for i in range(2, len(nodeCoords[e]), 3):
         nodeCoords[e][i] = 0
     gmsh.model.mesh.addNodes(e[0], e[1], nodeTags[e], nodeCoords[e])
-    gmsh.model.mesh.addElements(e[0], e[1], elementTypes[e], elementTags[e], elementNodeTags[e])
+    gmsh.model.mesh.addElements(e[0], e[1], elementTypes[e], elementTags[e],
+                                elementNodeTags[e])
 
-gmsh.fltk.run()
+if '-nopopup' not in sys.argv:
+    gmsh.fltk.run()
 #gmsh.write('flat.msh')
 
 gmsh.finalize()
