@@ -2,7 +2,7 @@ import gmsh
 import sys
 
 if len(sys.argv) < 2:
-    print "Usage: " + sys.argv[0] + " file.msh"
+    print("Usage: " + sys.argv[0] + " file.msh")
     exit(0)
 
 gmsh.initialize()
@@ -14,3 +14,4 @@ gmsh.model.mesh.optimize('', True)
 # gmsh.model.mesh.optimize('Netgen', True)
 
 gmsh.write('opt.msh')
+gmsh.finalize()
