@@ -6,9 +6,9 @@
 #
 # ------------------------------------------------------------------------------
 
-# Characteristic lengths can be specified very accurately by providing a
-# background mesh, i.e., a post-processing view that contains the target mesh
-# sizes.
+# As seen in `t7.py', characteristic lengths can be specified very accurately by
+# providing a background mesh, i.e., a post-processing view that contains the
+# target mesh sizes.
 
 # Here, the background mesh is represented as a metric tensor field defined on a
 # square. One should use bamg as 2d mesh generator to enable anisotropic meshes
@@ -43,5 +43,6 @@ gmsh.option.setNumber("Mesh.Algorithm", 7)
 gmsh.model.mesh.generate(2)
 gmsh.write("t17.msh")
 
-gmsh.fltk.run()
+# gmsh.fltk.run()
+
 gmsh.finalize()
